# Copyright (C) 2014, Red Hat, Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from unittest import mock

from oslo_utils.fixture import uuidsentinel as uuids

from nova.db.main import api as db
from nova import objects
from nova.objects import ec2 as ec2_obj
from nova.tests.unit.objects import test_objects


fake_map = {
    'created_at': None,
    'updated_at': None,
    'deleted_at': None,
    'deleted': 0,
    'id': 1,
    'uuid': uuids.ec2_map_uuid,
}


class _TestEC2InstanceMapping(object):
    @staticmethod
    def _compare(test, db, obj):
        for field, value in db.items():
            test.assertEqual(db[field], getattr(obj, field))

    def test_create(self):
        imap = ec2_obj.EC2InstanceMapping(context=self.context)
        imap.uuid = uuids.ec2_map_uuid

        with mock.patch.object(db, 'ec2_instance_create') as create:
            create.return_value = fake_map
            imap.create()

        self.assertEqual(self.context, imap._context)
        imap._context = None
        self._compare(self, fake_map, imap)

    def test_get_by_uuid(self):
        with mock.patch.object(db, 'ec2_instance_get_by_uuid') as get:
            get.return_value = fake_map
            imap = ec2_obj.EC2InstanceMapping.get_by_uuid(self.context,
                                                     uuids.ec2_map_uuid)
            self._compare(self, fake_map, imap)

    def test_get_by_ec2_id(self):
        with mock.patch.object(db, 'ec2_instance_get_by_id') as get:
            get.return_value = fake_map
            imap = ec2_obj.EC2InstanceMapping.get_by_id(self.context, 1)
            self._compare(self, fake_map, imap)


class TestEC2InstanceMapping(test_objects._LocalTest, _TestEC2InstanceMapping):
    pass


class TestRemoteEC2InstanceMapping(test_objects._RemoteTest,
                                   _TestEC2InstanceMapping):
    pass


class _TestS3ImageMapping(object):
    @staticmethod
    def _compare(test, db, obj):
        for field, value in db.items():
            test.assertEqual(db[field], getattr(obj, field))

    def test_create(self):
        s3imap = ec2_obj.S3ImageMapping(context=self.context)
        s3imap.uuid = uuids.ec2_map_uuid

        with mock.patch.object(db, 's3_image_create') as create:
            create.return_value = fake_map
            s3imap.create()

        self.assertEqual(self.context, s3imap._context)
        s3imap._context = None
        self._compare(self, fake_map, s3imap)

    def test_get_by_uuid(self):
        with mock.patch.object(db, 's3_image_get_by_uuid') as get:
            get.return_value = fake_map
            s3imap = ec2_obj.S3ImageMapping.get_by_uuid(self.context,
                                                        uuids.ec2_map_uuid)
            self._compare(self, fake_map, s3imap)

    def test_get_by_s3_id(self):
        with mock.patch.object(db, 's3_image_get') as get:
            get.return_value = fake_map
            s3imap = ec2_obj.S3ImageMapping.get_by_id(self.context, 1)
            self._compare(self, fake_map, s3imap)


class TestS3ImageMapping(test_objects._LocalTest, _TestS3ImageMapping):
    pass


class TestRemoteS3ImageMapping(test_objects._RemoteTest, _TestS3ImageMapping):
    pass


class _TestEC2Ids(object):
    @mock.patch('nova.objects.ec2.glance_type_to_ec2_type')
    @mock.patch('nova.objects.ec2.glance_id_to_ec2_id')
    @mock.patch('nova.objects.ec2.id_to_ec2_inst_id')
    def test_get_by_instance(self, mock_inst, mock_glance, mock_type):
        mock_inst.return_value = 'fake-ec2-inst-id'
        mock_glance.side_effect = ['fake-ec2-ami-id',
                                   'fake-ec2-kernel-id',
                                   'fake-ec2-ramdisk-id']
        mock_type.side_effect = [mock.sentinel.ec2_kernel_type,
                                 mock.sentinel.ec2_ramdisk_type]
        inst = objects.Instance(uuid=uuids.instance,
                                image_ref='fake-image-id',
                                kernel_id='fake-kernel-id',
                                ramdisk_id='fake-ramdisk-id')

        result = ec2_obj.EC2Ids.get_by_instance(self.context, inst)

        self.assertEqual('fake-ec2-inst-id', result.instance_id)
        self.assertEqual('fake-ec2-ami-id', result.ami_id)
        self.assertEqual('fake-ec2-kernel-id', result.kernel_id)
        self.assertEqual('fake-ec2-ramdisk-id', result.ramdisk_id)

    @mock.patch('nova.objects.ec2.glance_id_to_ec2_id')
    @mock.patch('nova.objects.ec2.id_to_ec2_inst_id')
    def test_get_by_instance_no_image_ref(self, mock_inst, mock_glance):
        mock_inst.return_value = 'fake-ec2-inst-id'
        mock_glance.return_value = None
        inst = objects.Instance(uuid=uuids.instance, image_ref=None,
                                kernel_id=None, ramdisk_id=None)

        result = ec2_obj.EC2Ids.get_by_instance(self.context, inst)

        self.assertEqual('fake-ec2-inst-id', result.instance_id)
        self.assertIsNone(result.ami_id)
        self.assertIsNone(result.kernel_id)
        self.assertIsNone(result.ramdisk_id)

    @mock.patch('nova.objects.ec2.glance_type_to_ec2_type')
    @mock.patch('nova.objects.ec2.glance_id_to_ec2_id')
    @mock.patch('nova.objects.ec2.id_to_ec2_inst_id')
    def test_get_by_instance_no_kernel_id(self, mock_inst, mock_glance,
                                          mock_type):
        mock_inst.return_value = 'fake-ec2-inst-id'
        mock_glance.side_effect = ['fake-ec2-ami-id',
                                   'fake-ec2-ramdisk-id']
        mock_type.return_value = mock.sentinel.ec2_ramdisk_type
        inst = objects.Instance(uuid=uuids.instance,
                                image_ref='fake-image-id',
                                kernel_id=None, ramdisk_id='fake-ramdisk-id')

        result = ec2_obj.EC2Ids.get_by_instance(self.context, inst)

        self.assertEqual('fake-ec2-inst-id', result.instance_id)
        self.assertEqual('fake-ec2-ami-id', result.ami_id)
        self.assertIsNone(result.kernel_id)
        self.assertEqual('fake-ec2-ramdisk-id', result.ramdisk_id)

    @mock.patch('nova.objects.ec2.glance_type_to_ec2_type')
    @mock.patch('nova.objects.ec2.glance_id_to_ec2_id')
    @mock.patch('nova.objects.ec2.id_to_ec2_inst_id')
    def test_get_by_instance_no_ramdisk_id(self, mock_inst, mock_glance,
                                          mock_type):
        mock_inst.return_value = 'fake-ec2-inst-id'
        mock_glance.side_effect = ['fake-ec2-ami-id',
                                   'fake-ec2-kernel-id']
        mock_type.return_value = mock.sentinel.ec2_kernel_type
        inst = objects.Instance(uuid=uuids.instance,
                                image_ref='fake-image-id',
                                kernel_id='fake-kernel-id', ramdisk_id=None)

        result = ec2_obj.EC2Ids.get_by_instance(self.context, inst)

        self.assertEqual('fake-ec2-inst-id', result.instance_id)
        self.assertEqual('fake-ec2-ami-id', result.ami_id)
        self.assertEqual('fake-ec2-kernel-id', result.kernel_id)
        self.assertIsNone(result.ramdisk_id)


class TestEC2Ids(test_objects._LocalTest, _TestEC2Ids):
    pass


class TestRemoteEC2Ids(test_objects._RemoteTest, _TestEC2Ids):
    pass



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import logging
from nova.db.main import api as db
from nova import exception
from nova.objects import base
from nova.objects import fields

LOG = logging.getLogger(__name__)


@base.NovaObjectRegistry.register
class ShareMapping(base.NovaTimestampObject, base.NovaObject):
    # Version 1.0: Initial version
    VERSION = '1.0'

    fields = {
        'id': fields.IntegerField(read_only=True),
        'uuid': fields.UUIDField(nullable=False),
        'instance_uuid': fields.UUIDField(nullable=False),
        'share_id': fields.UUIDField(nullable=False),
        'status': fields.ShareMappingStatusField(),
        'tag': fields.StringField(nullable=False),
        'export_location': fields.StringField(nullable=False),
        'share_proto': fields.ShareMappingProtoField()
    }

    @staticmethod
    def _from_db_object(context, share_mapping, db_share_mapping):
        for field in share_mapping.fields:
            setattr(share_mapping, field, db_share_mapping[field])
        share_mapping._context = context
        share_mapping.obj_reset_changes()
        return share_mapping

    @base.remotable
    def save(self):
        db_share_mapping = db.share_mapping_update(
            self._context, self.uuid, self.instance_uuid, self.share_id,
            self.status, self.tag, self.export_location, self.share_proto)
        self._from_db_object(self._context, self, db_share_mapping)

    def create(self):
        LOG.info(
            "Associate share '%s' to instance '%s'.",
            self.share_id, self.instance_uuid)

        self.save()

    @base.remotable
    def delete(self):
        LOG.info(
            "Dissociate share '%s' from instance '%s'.",
            self.share_id,
            self.instance_uuid,
        )
        db.share_mapping_delete_by_instance_uuid_and_share_id(
            self._context, self.instance_uuid, self.share_id
        )

    def attach(self):
        LOG.info(
            "Share '%s' about to be attached to instance '%s'.",
            self.share_id, self.instance_uuid)

        self.status = fields.ShareMappingStatus.ACTIVE
        self.save()

    def detach(self):
        LOG.info(
            "Share '%s' about to be detached from instance '%s'.",
            self.share_id,
            self.instance_uuid,
        )
        self.status = fields.ShareMappingStatus.INACTIVE
        self.save()

    @base.remotable_classmethod
    def get_by_instance_uuid_and_share_id(
            cls, context, instance_uuid, share_id):
        """This query returns only one element as a share can be
        associated only one time to an instance.
        Note: the REST API prevent the user to create duplicate share
        mapping by raising an exception.ShareMappingAlreadyExists.
        """
        share_mapping = ShareMapping(context)
        db_share_mapping = db.share_mapping_get_by_instance_uuid_and_share_id(
            context, instance_uuid, share_id)
        if not db_share_mapping:
            raise exception.ShareNotFound(share_id=share_id)
        return ShareMapping._from_db_object(
                context,
                share_mapping,
                db_share_mapping)

    def get_share_host_provider(self):
        if not self.export_location:
            return None
        if self.share_proto == 'NFS':
            rhost, _ = self.export_location.strip().split(':')
        else:
            raise NotImplementedError()
        return rhost


@base.NovaObjectRegistry.register
class ShareMappingList(base.ObjectListBase, base.NovaObject):
    # Version 1.0: Initial version
    VERSION = '1.0'
    fields = {
        'objects': fields.ListOfObjectsField('ShareMapping'),
    }

    @base.remotable_classmethod
    def get_by_instance_uuid(cls, context, instance_uuid):
        db_share_mappings = db.share_mapping_get_by_instance_uuid(
            context, instance_uuid)
        return base.obj_make_list(
            context, cls(context), ShareMapping, db_share_mappings)

    @base.remotable_classmethod
    def get_by_share_id(cls, context, share_id):
        db_share_mappings = db.share_mapping_get_by_share_id(
            context, share_id)
        return base.obj_make_list(
            context, cls(context), ShareMapping, db_share_mappings)

    def attach_all(self):
        for share in self:
            share.attach()

    def detach_all(self):
        for share in self:
            share.detach()



# Copyright (c) 2015 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
"""
Disk Weigher.  Weigh hosts by their disk usage.

The default is to spread instances across all hosts evenly.  If you prefer
stacking, you can set the 'disk_weight_multiplier' option (by configuration
or aggregate metadata) to a negative number and the weighing has the opposite
effect of the default.
"""

import nova.conf
from nova.scheduler import utils
from nova.scheduler import weights

CONF = nova.conf.CONF


class DiskWeigher(weights.BaseHostWeigher):
    minval = 0

    def weight_multiplier(self, host_state):
        """Override the weight multiplier."""
        return utils.get_weight_multiplier(
            host_state, 'disk_weight_multiplier',
            CONF.filter_scheduler.disk_weight_multiplier)

    def _weigh_object(self, host_state, weight_properties):
        """Higher weights win.  We want spreading to be the default."""
        return host_state.free_disk_mb



# Copyright 2020, Red Hat, Inc. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

"""Display extra specs in documentation.

Provides a single directive that can be used to list all extra specs validators
and, thus, document all extra specs that nova recognizes and supports.
"""

import typing as ty

from docutils import nodes
from docutils.parsers import rst
from docutils.parsers.rst import directives
from docutils import statemachine
from sphinx import addnodes
from sphinx import directives as sphinx_directives
from sphinx import domains
from sphinx import roles
from sphinx.util import logging
from sphinx.util import nodes as sphinx_nodes

from nova.api.validation.extra_specs import base
from nova.api.validation.extra_specs import validators

LOG = logging.getLogger(__name__)


class ExtraSpecXRefRole(roles.XRefRole):
    """Cross reference a extra spec.

    Example::

        :nova:extra-spec:`hw:cpu_policy`
    """

    def __init__(self):
        super(ExtraSpecXRefRole, self).__init__(
            warn_dangling=True,
        )

    def process_link(self, env, refnode, has_explicit_title, title, target):
        # The anchor for the extra spec link is the extra spec name
        return target, target


class ExtraSpecDirective(sphinx_directives.ObjectDescription):
    """Document an individual extra spec.

    Accepts one required argument - the extra spec name, including the group.

    Example::

        .. extra-spec:: hw:cpu_policy
    """

    def handle_signature(self, sig, signode):
        """Transform an option description into RST nodes."""
        # Insert a node into the output showing the extra spec name
        signode += addnodes.desc_name(sig, sig)
        signode['allnames'] = [sig]
        return sig

    def add_target_and_index(self, firstname, sig, signode):
        cached_options = self.env.domaindata['nova']['extra_specs']
        signode['ids'].append(sig)
        self.state.document.note_explicit_target(signode)
        # Store the location of the option definition for later use in
        # resolving cross-references
        cached_options[sig] = self.env.docname


def _indent(text, count=1):
    if not text:
        return text

    padding = ' ' * (4 * count)
    return padding + text


def _format_validator_group_help(
    validators: ty.Dict[str, base.ExtraSpecValidator],
    summary: bool,
):
    """Generate reStructuredText snippets for a group of validators."""
    for validator in validators.values():
        for line in _format_validator_help(validator, summary):
            yield line


def _format_validator_help(
    validator: base.ExtraSpecValidator,
    summary: bool,
):
    """Generate reStructuredText snippets for the provided validator.

    :param validator: A validator to document.
    :type validator: nova.api.validation.extra_specs.base.ExtraSpecValidator
    """
    yield f'.. nova:extra-spec:: {validator.name}'
    yield ''

    # NOTE(stephenfin): We don't print the pattern, if present, since it's too
    # internal. Instead, the description should provide this information in a
    # human-readable format
    yield _indent(f':Type: {validator.value["type"].__name__}')

    if validator.value.get('min') is not None:
        yield _indent(f':Min: {validator.value["min"]}')

    if validator.value.get('max') is not None:
        yield _indent(f':Max: {validator.value["max"]}')

    yield ''

    if not summary:
        for line in validator.description.splitlines():
            yield _indent(line)

        yield ''

    if validator.deprecated:
        yield _indent('.. warning::')
        yield _indent(
            'This extra spec has been deprecated and should not be used.', 2
        )
        yield ''


class ExtraSpecGroupDirective(rst.Directive):
    """Document extra specs belonging to the specified group.

    Accepts one optional argument - the extra spec group - and one option -
    whether to show a summary view only (omit descriptions). Example::

        .. extra-specs:: hw_rng
           :summary:
    """

    required_arguments = 0
    optional_arguments = 1
    option_spec = {
        'summary': directives.flag,
    }
    has_content = False

    def run(self):
        result = statemachine.ViewList()
        source_name = self.state.document.current_source

        group = self.arguments[0] if self.arguments else None
        summary = self.options.get('summary', False)

        if group:
            group_validators = {
                n.split(':', 1)[1]: v for n, v in validators.VALIDATORS.items()
                if ':' in n and n.split(':', 1)[0].split('{')[0] == group
            }
        else:
            group_validators = {
                n: v for n, v in validators.VALIDATORS.items()
                if ':' not in n
            }

        if not group_validators:
            LOG.warning("No validators found for group '%s'", group or '')

        for count, line in enumerate(
            _format_validator_group_help(group_validators, summary)
        ):
            result.append(line, source_name, count)
            LOG.debug('%5d%s%s', count, ' ' if line else '', line)

        node = nodes.section()
        node.document = self.state.document

        sphinx_nodes.nested_parse_with_titles(self.state, result, node)

        return node.children


class NovaDomain(domains.Domain):
    """nova domain."""
    name = 'nova'
    label = 'nova'
    object_types = {
        'configoption': domains.ObjType(
            'extra spec', 'spec',
        ),
    }
    directives = {
        'extra-spec': ExtraSpecDirective,
    }
    roles = {
        'extra-spec': ExtraSpecXRefRole(),
    }
    initial_data = {
        'extra_specs': {},
    }

    def resolve_xref(
        self, env, fromdocname, builder, typ, target, node, contnode,
    ):
        """Resolve cross-references"""
        if typ == 'extra-spec':
            return sphinx_nodes.make_refnode(
                builder,
                fromdocname,
                env.domaindata['nova']['extra_specs'][target],
                target,
                contnode,
                target,
            )
        return None

    def merge_domaindata(self, docnames, otherdata):
        for target, docname in otherdata['extra_specs'].items():
            if docname in docnames:
                self.data['extra_specs'][target] = docname


def setup(app):
    app.add_domain(NovaDomain)
    app.add_directive('extra-specs', ExtraSpecGroupDirective)
    return {
        'parallel_read_safe': True,
        'parallel_write_safe': True,
    }



#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
"""
Tests For Scheduler build failure weights.
"""

from nova import objects
from nova.scheduler import weights
from nova.scheduler.weights import compute
from nova import test
from nova.tests.unit.scheduler import fakes


class BuildFailureWeigherTestCase(test.NoDBTestCase):
    def setUp(self):
        super(BuildFailureWeigherTestCase, self).setUp()
        self.weight_handler = weights.HostWeightHandler()
        self.weighers = [compute.BuildFailureWeigher()]
        self.buildfailure_weigher = compute.BuildFailureWeigher()

    def _get_weighed_host(self, hosts):
        return self.weight_handler.get_weighed_objects(self.weighers,
                hosts, {})

    def _get_all_hosts(self):
        host_values = [
            ('host1', 'node1', {'failed_builds': 0}),
            ('host2', 'node2', {'failed_builds': 1}),
            ('host3', 'node3', {'failed_builds': 10}),
            ('host4', 'node4', {'failed_builds': 100})
        ]
        return [fakes.FakeHostState(host, node, values)
                for host, node, values in host_values]

    def test_build_failure_weigher_disabled(self):
        self.flags(build_failure_weight_multiplier=0.0,
                   group='filter_scheduler')
        hosts = self._get_all_hosts()
        weighed_hosts = self._get_weighed_host(hosts)
        self.assertTrue(all([wh.weight == 0.0
                             for wh in weighed_hosts]))

    def test_build_failure_weigher_scaled(self):
        self.flags(build_failure_weight_multiplier=1000.0,
                   group='filter_scheduler')
        hosts = self._get_all_hosts()
        weighed_hosts = self._get_weighed_host(hosts)
        self.assertEqual([0, -10, -100, -1000],
                         [wh.weight for wh in weighed_hosts])

    def test_build_failure_weight_multiplier(self):
        self.flags(build_failure_weight_multiplier=0.0,
                   group='filter_scheduler')
        host_attr = {'failed_builds': 1}
        host1 = fakes.FakeHostState('fake-host', 'node', host_attr)
        # By default, return the weight_multiplier configuration directly
        self.assertEqual(0.0,
                         self.buildfailure_weigher.weight_multiplier(host1))

        host1.aggregates = [
            objects.Aggregate(
                id=1,
                name='foo',
                hosts=['fake-host'],
                metadata={'build_failure_weight_multiplier': '1000.0'},
            )]
        # read the weight multiplier from metadata to override the config
        self.assertEqual(-1000,
                         self.buildfailure_weigher.weight_multiplier(host1))

        host1.aggregates = [
            objects.Aggregate(
                id=1,
                name='foo',
                hosts=['fake-host'],
                metadata={'build_failure_weight_multiplier': '500'},
            ),
            objects.Aggregate(
                id=2,
                name='foo',
                hosts=['fake-host'],
                metadata={'build_failure_weight_multiplier': '1000'},
            )]
        # If the host is in multiple aggs and there are conflict weight values
        # in the metadata, we will use the min value among them
        self.assertEqual(-500,
                         self.buildfailure_weigher.weight_multiplier(host1))

    def test_host_with_agg(self):
        self.flags(build_failure_weight_multiplier=0.0,
                   group='filter_scheduler')
        hostinfo_list = self._get_all_hosts()
        aggs = [
            objects.Aggregate(
                id=1,
                name='foo',
                hosts=['host1', 'host2', 'host3', 'host4'],
                metadata={'build_failure_weight_multiplier': '1000'},
            )]
        for h in hostinfo_list:
            h.aggregates = aggs

        weights = self.weight_handler.get_weighed_objects(self.weighers,
                                                          hostinfo_list, {})
        self.assertEqual([0, -10, -100, -1000],
                         [wh.weight for wh in weights])



# Copyright (c) 2012 Midokura Japan K.K.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from unittest import mock

from oslo_utils.fixture import uuidsentinel as uuids
import webob

from nova.api.openstack.compute import servers
from nova.compute import api as compute_api
from nova.db.main import api as db
from nova import exception
from nova import test
from nova.tests import fixtures as nova_fixtures
from nova.tests.unit.api.openstack import fakes


class ServerStartStopTestV21(test.TestCase):

    def setUp(self):
        super(ServerStartStopTestV21, self).setUp()
        self._setup_controller()
        self.req = fakes.HTTPRequest.blank('')
        self.useFixture(nova_fixtures.SingleCellSimple())
        self.stub_out('nova.db.main.api.instance_get_by_uuid',
                      fakes.fake_instance_get(
                          project_id=fakes.FAKE_PROJECT_ID))

    def _setup_controller(self):
        self.controller = servers.ServersController()

    @mock.patch.object(compute_api.API, 'start')
    def test_start(self, start_mock):
        body = dict(start="")
        self.controller._start_server(self.req, uuids.instance, body)
        start_mock.assert_called_once_with(mock.ANY, mock.ANY)

    @mock.patch.object(compute_api.API, 'start',
                       side_effect=exception.InstanceNotReady(
                           instance_id=uuids.instance))
    def test_start_not_ready(self, start_mock):
        body = dict(start="")
        self.assertRaises(webob.exc.HTTPConflict,
            self.controller._start_server, self.req, uuids.instance, body)

    @mock.patch.object(compute_api.API, 'start',
                       side_effect=exception.InstanceIsLocked(
                           instance_uuid=uuids.instance))
    def test_start_locked_server(self, start_mock):
        body = dict(start="")
        self.assertRaises(webob.exc.HTTPConflict,
            self.controller._start_server, self.req, uuids.instance, body)

    @mock.patch.object(compute_api.API, 'start',
                       side_effect=exception.InstanceIsLocked(
                           instance_uuid=uuids.instance))
    def test_start_invalid_state(self, start_mock):
        body = dict(start="")
        ex = self.assertRaises(webob.exc.HTTPConflict,
            self.controller._start_server, self.req, uuids.instance, body)
        self.assertIn('is locked', str(ex))

    @mock.patch.object(compute_api.API, 'stop')
    def test_stop(self, stop_mock):
        body = dict(stop="")
        self.controller._stop_server(self.req, uuids.instance, body)
        stop_mock.assert_called_once_with(mock.ANY, mock.ANY)

    @mock.patch.object(compute_api.API, 'stop',
                       side_effect=exception.InstanceNotReady(
                           instance_id=uuids.instance))
    def test_stop_not_ready(self, stop_mock):
        body = dict(stop="")
        self.assertRaises(webob.exc.HTTPConflict,
            self.controller._stop_server, self.req, uuids.instance, body)

    @mock.patch.object(compute_api.API, 'stop',
                       side_effect=exception.InstanceIsLocked(
                           instance_uuid=uuids.instance))
    def test_stop_locked_server(self, stop_mock):
        body = dict(stop="")
        ex = self.assertRaises(webob.exc.HTTPConflict,
            self.controller._stop_server, self.req, uuids.instance, body)
        self.assertIn('is locked', str(ex))

    @mock.patch.object(compute_api.API, 'stop',
                       side_effect=exception.InstanceIsLocked(
                           instance_uuid=uuids.instance))
    def test_stop_invalid_state(self, stop_mock):
        body = dict(start="")
        self.assertRaises(webob.exc.HTTPConflict,
            self.controller._stop_server, self.req, uuids.instance, body)

    @mock.patch.object(db, 'instance_get_by_uuid',
                       side_effect=exception.InstanceNotFound(
                           instance_id=uuids.instance))
    def test_start_with_bogus_id(self, get_mock):
        body = dict(start="")
        self.assertRaises(webob.exc.HTTPNotFound,
            self.controller._start_server, self.req, uuids.instance, body)

    @mock.patch.object(db, 'instance_get_by_uuid',
                       side_effect=exception.InstanceNotFound(
                           instance_id=uuids.instance))
    def test_stop_with_bogus_id(self, get_mock):
        body = dict(stop="")
        self.assertRaises(webob.exc.HTTPNotFound,
            self.controller._stop_server, self.req, uuids.instance, body)



# Copyright 2011 OpenStack Foundation
# Copyright 2013 IBM Corp.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from unittest import mock

from oslo_limit import fixture as limit_fixture
from oslo_utils.fixture import uuidsentinel as uuids
import webob

from nova.api.openstack.compute import quota_sets as quotas_v21
from nova.db import constants as db_const
from nova import exception
from nova.limit import local as local_limit
from nova.limit import placement as placement_limit
from nova import objects
from nova import quota
from nova import test
from nova.tests.unit.api.openstack import fakes


def quota_set(id, include_server_group_quotas=True):
    res = {'quota_set': {'id': id, 'metadata_items': 128,
           'ram': 51200, 'floating_ips': -1, 'fixed_ips': -1,
           'instances': 10, 'injected_files': 5, 'cores': 20,
           'injected_file_content_bytes': 10240,
           'security_groups': -1, 'security_group_rules': -1,
           'key_pairs': 100, 'injected_file_path_bytes': 255}}
    if include_server_group_quotas:
        res['quota_set']['server_groups'] = 10
        res['quota_set']['server_group_members'] = 10
    return res


class BaseQuotaSetsTest(test.TestCase):

    def setUp(self):
        super(BaseQuotaSetsTest, self).setUp()
        # We need to stub out verify_project_id so that it doesn't
        # generate an EndpointNotFound exception and result in a
        # server error.
        self.stub_out('nova.api.openstack.identity.verify_project_id',
                      lambda ctx, project_id: True)

    def get_delete_status_int(self, res):
        # NOTE: on v2.1, http status code is set as wsgi_code of API
        # method instead of status_int in a response object.
        return self.controller.delete.wsgi_code


class QuotaSetsTestV21(BaseQuotaSetsTest):
    plugin = quotas_v21
    validation_error = exception.ValidationError
    include_server_group_quotas = True

    def setUp(self):
        super(QuotaSetsTestV21, self).setUp()
        self._setup_controller()
        self.default_quotas = {
            'instances': 10,
            'cores': 20,
            'ram': 51200,
            'floating_ips': -1,
            'fixed_ips': -1,
            'metadata_items': 128,
            'injected_files': 5,
            'injected_file_path_bytes': 255,
            'injected_file_content_bytes': 10240,
            'security_groups': -1,
            'security_group_rules': -1,
            'key_pairs': 100,
        }
        if self.include_server_group_quotas:
            self.default_quotas['server_groups'] = 10
            self.default_quotas['server_group_members'] = 10

    def _setup_controller(self):
        self.controller = self.plugin.QuotaSetsController()

    def _get_http_request(self, url=''):
        return fakes.HTTPRequest.blank(url)

    def test_format_quota_set(self):
        quota_set = self.controller._format_quota_set('1234',
                                                      self.default_quotas,
                                                      [])
        qs = quota_set['quota_set']

        self.assertEqual(qs['id'], '1234')
        self.assertEqual(qs['instances'], 10)
        self.assertEqual(qs['cores'], 20)
        self.assertEqual(qs['ram'], 51200)
        self.assertEqual(qs['floating_ips'], -1)
        self.assertEqual(qs['fixed_ips'], -1)
        self.assertEqual(qs['metadata_items'], 128)
        self.assertEqual(qs['injected_files'], 5)
        self.assertEqual(qs['injected_file_path_bytes'], 255)
        self.assertEqual(qs['injected_file_content_bytes'], 10240)
        self.assertEqual(qs['security_groups'], -1)
        self.assertEqual(qs['security_group_rules'], -1)
        self.assertEqual(qs['key_pairs'], 100)
        if self.include_server_group_quotas:
            self.assertEqual(qs['server_groups'], 10)
            self.assertEqual(qs['server_group_members'], 10)

    def test_validate_quota_limit(self):
        resource = 'fake'

        # Valid - finite values
        self.assertIsNone(self.controller._validate_quota_limit(resource,
                                                                50, 10, 100))

        # Valid - finite limit and infinite maximum
        self.assertIsNone(self.controller._validate_quota_limit(resource,
                                                                50, 10, -1))

        # Valid - infinite limit and infinite maximum
        self.assertIsNone(self.controller._validate_quota_limit(resource,
                                                                -1, 10, -1))

        # Valid - all infinite
        self.assertIsNone(self.controller._validate_quota_limit(resource,
                                                                -1, -1, -1))

        # Invalid - limit is less than -1
        self.assertRaises(webob.exc.HTTPBadRequest,
                          self.controller._validate_quota_limit,
                          resource, -2, 10, 100)

        # Invalid - limit is less than minimum
        self.assertRaises(webob.exc.HTTPBadRequest,
                          self.controller._validate_quota_limit,
                          resource, 5, 10, 100)

        # Invalid - limit is greater than maximum
        self.assertRaises(webob.exc.HTTPBadRequest,
                          self.controller._validate_quota_limit,
                          resource, 200, 10, 100)

        # Invalid - infinite limit is greater than maximum
        self.assertRaises(webob.exc.HTTPBadRequest,
                          self.controller._validate_quota_limit,
                          resource, -1, 10, 100)

        # Invalid - limit is less than infinite minimum
        self.assertRaises(webob.exc.HTTPBadRequest,
                          self.controller._validate_quota_limit,
                          resource, 50, -1, -1)

        # Invalid - limit is larger than 0x7FFFFFFF
        self.assertRaises(webob.exc.HTTPBadRequest,
                          self.controller._validate_quota_limit,
                          resource, db_const.MAX_INT + 1, -1, -1)

    def test_quotas_defaults(self):
        uri = '/v2/%s/os-quota-sets/%s/defaults' % (
                fakes.FAKE_PROJECT_ID, fakes.FAKE_PROJECT_ID)

        req = fakes.HTTPRequest.blank(uri)
        res_dict = self.controller.defaults(req, fakes.FAKE_PROJECT_ID)
        self.default_quotas.update({'id': fakes.FAKE_PROJECT_ID})
        expected = {'quota_set': self.default_quotas}

        self.assertEqual(res_dict, expected)

    def test_quotas_show(self):
        req = self._get_http_request()
        res_dict = self.controller.show(req, 1234)

        ref_quota_set = quota_set('1234', self.include_server_group_quotas)
        self.assertEqual(res_dict, ref_quota_set)

    def test_quotas_update(self):
        self.default_quotas.update({
            'instances': 50,
            'cores': 50
        })
        body = {'quota_set': self.default_quotas}
        req = self._get_http_request()
        res_dict = self.controller.update(req, 'update_me', body=body)
        self.assertEqual(body, res_dict)

    @mock.patch('nova.objects.Quotas.create_limit')
    def test_quotas_update_with_good_data(self, mock_createlimit):
        self.default_quotas.update({})
        body = {'quota_set': self.default_quotas}
        req = self._get_http_request()
        self.controller.update(req, 'update_me', body=body)
        self.assertEqual(len(self.default_quotas),
                         len(mock_createlimit.mock_calls))

    @mock.patch('nova.api.validation.validators._SchemaValidator.validate')
    @mock.patch('nova.objects.Quotas.create_limit')
    def test_quotas_update_with_bad_data(self, mock_createlimit,
                                                  mock_validate):
        self.default_quotas.update({
            'instances': 50,
            'cores': -50
        })
        body = {'quota_set': self.default_quotas}
        req = self._get_http_request()
        self.assertRaises(webob.exc.HTTPBadRequest, self.controller.update,
                          req, 'update_me', body=body)
        self.assertEqual(0,
                         len(mock_createlimit.mock_calls))

    def test_quotas_update_zero_value(self):
        body = {'quota_set': {'instances': 0, 'cores': 0,
                              'ram': 0, 'floating_ips': -1,
                              'metadata_items': 0,
                              'injected_files': 0,
                              'injected_file_content_bytes': 0,
                              'injected_file_path_bytes': 0,
                              'security_groups': -1,
                              'security_group_rules': -1,
                              'key_pairs': 100, 'fixed_ips': -1}}
        if self.include_server_group_quotas:
            body['quota_set']['server_groups'] = 10
            body['quota_set']['server_group_members'] = 10

        req = self._get_http_request()
        res_dict = self.controller.update(req, 'update_me', body=body)
        self.assertEqual(body, res_dict)

    def _quotas_update_bad_request_case(self, body):
        req = self._get_http_request()
        self.assertRaises(self.validation_error, self.controller.update,
                          req, 'update_me', body=body)

    def test_quotas_update_invalid_key(self):
        body = {'quota_set': {'instances2': -2, 'cores': -2,
                              'ram': -2, 'floating_ips': -2,
                              'metadata_items': -2, 'injected_files': -2,
                              'injected_file_content_bytes': -2}}
        self._quotas_update_bad_request_case(body)

    def test_quotas_update_invalid_limit(self):
        body = {'quota_set': {'instances': -2, 'cores': -2,
                              'ram': -2, 'floating_ips': -2, 'fixed_ips': -2,
                              'metadata_items': -2, 'injected_files': -2,
                              'injected_file_content_bytes': -2}}
        self._quotas_update_bad_request_case(body)

    def test_quotas_update_empty_body(self):
        body = {}
        self._quotas_update_bad_request_case(body)

    def test_quotas_update_invalid_value_non_int(self):
        # when PUT non integer value
        self.default_quotas.update({
            'instances': 'test'
        })
        body = {'quota_set': self.default_quotas}
        self._quotas_update_bad_request_case(body)

    def test_quotas_update_invalid_value_with_float(self):
        # when PUT non integer value
        self.default_quotas.update({
            'instances': 50.5
        })
        body = {'quota_set': self.default_quotas}
        self._quotas_update_bad_request_case(body)

    def test_quotas_update_invalid_value_with_unicode(self):
        # when PUT non integer value
        self.default_quotas.update({
            'instances': u'\u30aa\u30fc\u30d7\u30f3'
        })
        body = {'quota_set': self.default_quotas}
        self._quotas_update_bad_request_case(body)

    @mock.patch('nova.objects.Quotas.destroy_all_by_project')
    def test_quotas_delete(self, mock_destroy_all_by_project):
        req = self._get_http_request()
        res = self.controller.delete(req, 1234)
        self.assertEqual(202, self.get_delete_status_int(res))
        mock_destroy_all_by_project.assert_called_once_with(
            req.environ['nova.context'], 1234)

    def test_duplicate_quota_filter(self):
        query_string = 'user_id=1&user_id=2'
        req = fakes.HTTPRequest.blank('', query_string=query_string)
        self.controller.show(req, 1234)
        self.controller.update(req, 1234, body={'quota_set': {}})
        self.controller.detail(req, 1234)
        self.controller.delete(req, 1234)

    def test_quota_filter_negative_int_as_string(self):
        req = fakes.HTTPRequest.blank('', query_string='user_id=-1')
        self.controller.show(req, 1234)
        self.controller.update(req, 1234, body={'quota_set': {}})
        self.controller.detail(req, 1234)
        self.controller.delete(req, 1234)

    def test_quota_filter_int_as_string(self):
        req = fakes.HTTPRequest.blank('', query_string='user_id=123')
        self.controller.show(req, 1234)
        self.controller.update(req, 1234, body={'quota_set': {}})
        self.controller.detail(req, 1234)
        self.controller.delete(req, 1234)

    def test_unknown_quota_filter(self):
        query_string = 'unknown_filter=abc'
        req = fakes.HTTPRequest.blank('', query_string=query_string)
        self.controller.show(req, 1234)
        self.controller.update(req, 1234, body={'quota_set': {}})
        self.controller.detail(req, 1234)
        self.controller.delete(req, 1234)

    def test_quota_additional_filter(self):
        query_string = 'user_id=1&additional_filter=2'
        req = fakes.HTTPRequest.blank('', query_string=query_string)
        self.controller.show(req, 1234)
        self.controller.update(req, 1234, body={'quota_set': {}})
        self.controller.detail(req, 1234)
        self.controller.delete(req, 1234)


class ExtendedQuotasTestV21(BaseQuotaSetsTest):
    plugin = quotas_v21

    def setUp(self):
        super(ExtendedQuotasTestV21, self).setUp()
        self._setup_controller()

    fake_quotas = {'ram': {'limit': 51200,
                           'in_use': 12800,
                           'reserved': 12800},
                   'cores': {'limit': 20,
                             'in_use': 10,
                             'reserved': 5},
                   'instances': {'limit': 100,
                                 'in_use': 0,
                                 'reserved': 0}}

    def _setup_controller(self):
        self.controller = self.plugin.QuotaSetsController()

    def fake_get_quotas(self, context, id, user_id=None, usages=False):
        if usages:
            return self.fake_quotas
        else:
            return {k: v['limit'] for k, v in self.fake_quotas.items()}

    def fake_get_settable_quotas(self, context, project_id, user_id=None):
        return {
            'ram': {'minimum': self.fake_quotas['ram']['in_use'] +
                               self.fake_quotas['ram']['reserved'],
                    'maximum': -1},
            'cores': {'minimum': self.fake_quotas['cores']['in_use'] +
                                 self.fake_quotas['cores']['reserved'],
                      'maximum': -1},
            'instances': {'minimum': self.fake_quotas['instances']['in_use'] +
                                     self.fake_quotas['instances']['reserved'],
                          'maximum': -1},
        }

    def _get_http_request(self, url=''):
        return fakes.HTTPRequest.blank(url)

    @mock.patch.object(quota.QUOTAS, 'get_settable_quotas')
    def test_quotas_update_exceed_in_used(self, get_settable_quotas):
        body = {'quota_set': {'cores': 10}}

        get_settable_quotas.side_effect = self.fake_get_settable_quotas
        req = self._get_http_request()
        self.assertRaises(webob.exc.HTTPBadRequest, self.controller.update,
                          req, 'update_me', body=body)

    @mock.patch.object(quota.QUOTAS, 'get_settable_quotas')
    def test_quotas_force_update_exceed_in_used(self, get_settable_quotas):
        with mock.patch.object(self.plugin.QuotaSetsController,
                               '_get_quotas') as _get_quotas:

            body = {'quota_set': {'cores': 10, 'force': 'True'}}

            get_settable_quotas.side_effect = self.fake_get_settable_quotas
            _get_quotas.side_effect = self.fake_get_quotas
            req = self._get_http_request()
            self.controller.update(req, 'update_me', body=body)

    @mock.patch('nova.objects.Quotas.create_limit')
    def test_quotas_update_good_data(self, mock_createlimit):
        body = {'quota_set': {'cores': 1,
                              'instances': 1}}
        req = fakes.HTTPRequest.blank(
                '/v2/%s/os-quota-sets/update_me' % fakes.FAKE_PROJECT_ID,
                use_admin_context=True)
        self.controller.update(req, 'update_me', body=body)
        self.assertEqual(2,
                         len(mock_createlimit.mock_calls))

    @mock.patch('nova.objects.Quotas.create_limit')
    @mock.patch.object(quota.QUOTAS, 'get_settable_quotas')
    def test_quotas_update_bad_data(self, mock_gsq, mock_createlimit):
        body = {'quota_set': {'cores': 10,
                              'instances': 1}}

        mock_gsq.side_effect = self.fake_get_settable_quotas
        req = fakes.HTTPRequest.blank(
                '/v2/%s/os-quota-sets/update_me' % fakes.FAKE_PROJECT_ID,
                use_admin_context=True)
        self.assertRaises(webob.exc.HTTPBadRequest, self.controller.update,
                          req, 'update_me', body=body)
        self.assertEqual(0,
                         len(mock_createlimit.mock_calls))


class UserQuotasTestV21(BaseQuotaSetsTest):
    plugin = quotas_v21
    include_server_group_quotas = True

    def setUp(self):
        super(UserQuotasTestV21, self).setUp()
        self._setup_controller()

    def _get_http_request(self, url=''):
        return fakes.HTTPRequest.blank(url)

    def _setup_controller(self):
        self.controller = self.plugin.QuotaSetsController()

    def test_user_quotas_show(self):
        req = self._get_http_request(
                '/v2/%s/os-quota-sets/%s?user_id=1' % (fakes.FAKE_PROJECT_ID,
                                                       fakes.FAKE_PROJECT_ID))
        res_dict = self.controller.show(req, fakes.FAKE_PROJECT_ID)
        ref_quota_set = quota_set(fakes.FAKE_PROJECT_ID,
                                  self.include_server_group_quotas)
        self.assertEqual(res_dict, ref_quota_set)

    def test_user_quotas_update(self):
        body = {'quota_set': {'instances': 10, 'cores': 20,
                              'ram': 51200, 'floating_ips': -1,
                              'fixed_ips': -1, 'metadata_items': 128,
                              'injected_files': 5,
                              'injected_file_content_bytes': 10240,
                              'injected_file_path_bytes': 255,
                              'security_groups': -1,
                              'security_group_rules': -1,
                              'key_pairs': 100}}
        if self.include_server_group_quotas:
            body['quota_set']['server_groups'] = 10
            body['quota_set']['server_group_members'] = 10

        url = ('/v2/%s/os-quota-sets/update_me?user_id=1' %
               fakes.FAKE_PROJECT_ID)
        req = self._get_http_request(url)
        res_dict = self.controller.update(req, 'update_me', body=body)

        self.assertEqual(body, res_dict)

    def test_user_quotas_update_exceed_project(self):
        body = {'quota_set': {'instances': 20}}

        url = ('/v2/%s/os-quota-sets/update_me?user_id=1' %
               fakes.FAKE_PROJECT_ID)
        req = self._get_http_request(url)
        self.assertRaises(webob.exc.HTTPBadRequest, self.controller.update,
                          req, 'update_me', body=body)

    @mock.patch('nova.objects.Quotas.destroy_all_by_project_and_user')
    def test_user_quotas_delete(self, mock_destroy_all_by_project_and_user):
        url = '/v2/%s/os-quota-sets/%s?user_id=1' % (fakes.FAKE_PROJECT_ID,
                                                     fakes.FAKE_PROJECT_ID)
        req = self._get_http_request(url)
        res = self.controller.delete(req, fakes.FAKE_PROJECT_ID)
        self.assertEqual(202, self.get_delete_status_int(res))
        mock_destroy_all_by_project_and_user.assert_called_once_with(
            req.environ['nova.context'], fakes.FAKE_PROJECT_ID, '1'
        )

    @mock.patch('nova.objects.Quotas.create_limit')
    def test_user_quotas_update_good_data(self, mock_createlimit):
        body = {'quota_set': {'instances': 1,
                              'cores': 1}}

        url = ('/v2/%s/os-quota-sets/update_me?user_id=1' %
               fakes.FAKE_PROJECT_ID)
        req = fakes.HTTPRequest.blank(url, use_admin_context=True)
        self.controller.update(req, 'update_me', body=body)
        self.assertEqual(2,
                         len(mock_createlimit.mock_calls))

    @mock.patch('nova.objects.Quotas.create_limit')
    def test_user_quotas_update_bad_data(self, mock_createlimit):
        body = {'quota_set': {'instances': 20,
                              'cores': 1}}

        url = ('/v2/%s/os-quota-sets/update_me?user_id=1' %
               fakes.FAKE_PROJECT_ID)
        req = fakes.HTTPRequest.blank(url, use_admin_context=True)
        self.assertRaises(webob.exc.HTTPBadRequest, self.controller.update,
                          req, 'update_me', body=body)
        self.assertEqual(0,
                         len(mock_createlimit.mock_calls))


class QuotaSetsTestV236(test.NoDBTestCase):
    microversion = '2.36'

    def setUp(self):
        super(QuotaSetsTestV236, self).setUp()
        # We need to stub out verify_project_id so that it doesn't
        # generate an EndpointNotFound exception and result in a
        # server error.
        self.stub_out('nova.api.openstack.identity.verify_project_id',
                      lambda ctx, project_id: True)

        self.old_req = fakes.HTTPRequest.blank('', version='2.1')
        self.filtered_quotas = ['fixed_ips', 'floating_ips',
            'security_group_rules', 'security_groups']
        self.quotas = {
            'cores': {'limit': 20},
            'fixed_ips': {'limit': -1},
            'floating_ips': {'limit': -1},
            'injected_file_content_bytes': {'limit': 10240},
            'injected_file_path_bytes': {'limit': 255},
            'injected_files': {'limit': 5},
            'instances': {'limit': 10},
            'key_pairs': {'limit': 100},
            'metadata_items': {'limit': 128},
            'ram': {'limit': 51200},
            'security_group_rules': {'limit': -1},
            'security_groups': {'limit': -1},
            'server_group_members': {'limit': 10},
            'server_groups': {'limit': 10}
        }
        self.defaults = {
            'cores': 20,
            'fixed_ips': -1,
            'floating_ips': -1,
            'injected_file_content_bytes': 10240,
            'injected_file_path_bytes': 255,
            'injected_files': 5,
            'instances': 10,
            'key_pairs': 100,
            'metadata_items': 128,
            'ram': 51200,
            'security_group_rules': -1,
            'security_groups': -1,
            'server_group_members': 10,
            'server_groups': 10
        }
        self.controller = quotas_v21.QuotaSetsController()
        self.req = fakes.HTTPRequest.blank('', version=self.microversion)

    def _ensure_filtered_quotas_existed_in_old_api(self):
        res_dict = self.controller.show(self.old_req, 1234)
        for filtered in self.filtered_quotas:
            self.assertIn(filtered, res_dict['quota_set'])

    @mock.patch('nova.quota.QUOTAS.get_project_quotas')
    def test_quotas_show_filtered(self, mock_quotas):
        mock_quotas.return_value = self.quotas
        self._ensure_filtered_quotas_existed_in_old_api()
        res_dict = self.controller.show(self.req, 1234)
        for filtered in self.filtered_quotas:
            self.assertNotIn(filtered, res_dict['quota_set'])

    @mock.patch('nova.quota.QUOTAS.get_defaults')
    @mock.patch('nova.quota.QUOTAS.get_project_quotas')
    def test_quotas_default_filtered(self, mock_quotas, mock_defaults):
        mock_quotas.return_value = self.quotas
        self._ensure_filtered_quotas_existed_in_old_api()
        res_dict = self.controller.defaults(self.req, 1234)
        for filtered in self.filtered_quotas:
            self.assertNotIn(filtered, res_dict['quota_set'])

    @mock.patch('nova.quota.QUOTAS.get_project_quotas')
    def test_quotas_detail_filtered(self, mock_quotas):
        mock_quotas.return_value = self.quotas
        self._ensure_filtered_quotas_existed_in_old_api()
        res_dict = self.controller.detail(self.req, 1234)
        for filtered in self.filtered_quotas:
            self.assertNotIn(filtered, res_dict['quota_set'])

    @mock.patch('nova.quota.QUOTAS.get_project_quotas')
    def test_quotas_update_input_filtered(self, mock_quotas):
        mock_quotas.return_value = self.quotas
        self._ensure_filtered_quotas_existed_in_old_api()
        for filtered in self.filtered_quotas:
            self.assertRaises(exception.ValidationError,
                self.controller.update, self.req, 1234,
                body={'quota_set': {filtered: 100}})

    @mock.patch('nova.objects.Quotas.create_limit')
    @mock.patch('nova.quota.QUOTAS.get_settable_quotas')
    @mock.patch('nova.quota.QUOTAS.get_project_quotas')
    def test_quotas_update_output_filtered(self, mock_quotas, mock_settable,
                                           mock_create_limit):
        mock_quotas.return_value = self.quotas
        mock_settable.return_value = {'cores': {'maximum': -1, 'minimum': 0}}
        self._ensure_filtered_quotas_existed_in_old_api()
        res_dict = self.controller.update(self.req, 1234,
             body={'quota_set': {'cores': 100}})
        for filtered in self.filtered_quotas:
            self.assertNotIn(filtered, res_dict['quota_set'])


class QuotaSetsTestV257(QuotaSetsTestV236):
    microversion = '2.57'

    def setUp(self):
        super(QuotaSetsTestV257, self).setUp()
        self.filtered_quotas.extend(quotas_v21.FILTERED_QUOTAS_2_57)


class QuotaSetsTestV275(QuotaSetsTestV257):
    microversion = '2.75'

    @mock.patch('nova.objects.Quotas.destroy_all_by_project')
    @mock.patch('nova.objects.Quotas.create_limit')
    @mock.patch('nova.quota.QUOTAS.get_settable_quotas')
    @mock.patch('nova.quota.QUOTAS.get_project_quotas')
    def test_quota_additional_filter_older_version(self, mock_quotas,
                                                   mock_settable,
                                                   mock_create_limit,
                                                   mock_destroy):
        mock_quotas.return_value = self.quotas
        mock_settable.return_value = {'cores': {'maximum': -1, 'minimum': 0}}
        query_string = 'additional_filter=2'
        req = fakes.HTTPRequest.blank('', version='2.74',
                                      query_string=query_string)
        self.controller.show(req, 1234)
        self.controller.update(req, 1234, body={'quota_set': {}})
        self.controller.detail(req, 1234)
        self.controller.delete(req, 1234)

    def test_quota_update_additional_filter(self):
        query_string = 'user_id=1&additional_filter=2'
        req = fakes.HTTPRequest.blank('', version=self.microversion,
                                      query_string=query_string)
        self.assertRaises(exception.ValidationError, self.controller.update,
                          req, 'update_me', body={'quota_set': {}})

    def test_quota_show_additional_filter(self):
        query_string = 'user_id=1&additional_filter=2'
        req = fakes.HTTPRequest.blank('', version=self.microversion,
                                      query_string=query_string)
        self.assertRaises(exception.ValidationError, self.controller.show,
                          req, 1234)

    def test_quota_detail_additional_filter(self):
        query_string = 'user_id=1&additional_filter=2'
        req = fakes.HTTPRequest.blank('', version=self.microversion,
                                      query_string=query_string)
        self.assertRaises(exception.ValidationError, self.controller.detail,
                          req, 1234)

    def test_quota_delete_additional_filter(self):
        query_string = 'user_id=1&additional_filter=2'
        req = fakes.HTTPRequest.blank('', version=self.microversion,
                                      query_string=query_string)
        self.assertRaises(exception.ValidationError, self.controller.delete,
                          req, 1234)


class NoopQuotaSetsTest(test.NoDBTestCase):
    quota_driver = "nova.quota.NoopQuotaDriver"
    expected_detail = {'in_use': -1, 'limit': -1, 'reserved': -1}

    def setUp(self):
        super(NoopQuotaSetsTest, self).setUp()
        self.flags(driver=self.quota_driver, group="quota")
        self.controller = quotas_v21.QuotaSetsController()
        self.stub_out('nova.api.openstack.identity.verify_project_id',
                      lambda ctx, project_id: True)

    def test_show_v21(self):
        req = fakes.HTTPRequest.blank("")
        response = self.controller.show(req, uuids.project_id)
        expected_response = {
            'quota_set': {
                'id': uuids.project_id,
                'cores': -1,
                'fixed_ips': -1,
                'floating_ips': -1,
                'injected_file_content_bytes': -1,
                'injected_file_path_bytes': -1,
                'injected_files': -1,
                'instances': -1,
                'key_pairs': -1,
                'metadata_items': -1,
                'ram': -1,
                'security_group_rules': -1,
                'security_groups': -1,
                'server_group_members': -1,
                'server_groups': -1,
            }
        }
        self.assertEqual(expected_response, response)

    def test_show_v257(self):
        req = fakes.HTTPRequest.blank("", version='2.57')
        response = self.controller.show(req, uuids.project_id)
        expected_response = {
            'quota_set': {
                'id': uuids.project_id,
                'cores': -1,
                'instances': -1,
                'key_pairs': -1,
                'metadata_items': -1,
                'ram': -1,
                'server_group_members': -1,
                'server_groups': -1}}
        self.assertEqual(expected_response, response)

    def test_detail_v21(self):
        req = fakes.HTTPRequest.blank("")
        response = self.controller.detail(req, uuids.project_id)

        expected_response = {
            'quota_set': {
                'id': uuids.project_id,
                'cores': self.expected_detail,
                'fixed_ips': self.expected_detail,
                'floating_ips': self.expected_detail,
                'injected_file_content_bytes': self.expected_detail,
                'injected_file_path_bytes': self.expected_detail,
                'injected_files': self.expected_detail,
                'instances': self.expected_detail,
                'key_pairs': self.expected_detail,
                'metadata_items': self.expected_detail,
                'ram': self.expected_detail,
                'security_group_rules': self.expected_detail,
                'security_groups': self.expected_detail,
                'server_group_members': self.expected_detail,
                'server_groups': self.expected_detail,
            }
        }
        self.assertEqual(expected_response, response)

    def test_detail_v21_user(self):
        req = fakes.HTTPRequest.blank("?user_id=42")
        response = self.controller.detail(req, uuids.project_id)
        expected_response = {
            'quota_set': {
                'id': uuids.project_id,
                'cores': self.expected_detail,
                'fixed_ips': self.expected_detail,
                'floating_ips': self.expected_detail,
                'injected_file_content_bytes': self.expected_detail,
                'injected_file_path_bytes': self.expected_detail,
                'injected_files': self.expected_detail,
                'instances': self.expected_detail,
                'key_pairs': self.expected_detail,
                'metadata_items': self.expected_detail,
                'ram': self.expected_detail,
                'security_group_rules': self.expected_detail,
                'security_groups': self.expected_detail,
                'server_group_members': self.expected_detail,
                'server_groups': self.expected_detail,
            }
        }
        self.assertEqual(expected_response, response)

    def test_update_still_rejects_badrequests(self):
        req = fakes.HTTPRequest.blank("")
        body = {'quota_set': {'instances': 50, 'cores': 50,
                              'ram': 51200, 'unsupported': 12}}
        self.assertRaises(exception.ValidationError, self.controller.update,
                          req, uuids.project_id, body=body)

    @mock.patch.object(objects.Quotas, "create_limit")
    def test_update_v21(self, mock_create):
        req = fakes.HTTPRequest.blank("")
        body = {'quota_set': {'server_groups': 2}}
        response = self.controller.update(req, uuids.project_id, body=body)
        expected_response = {
            'quota_set': {
                'cores': -1,
                'fixed_ips': -1,
                'floating_ips': -1,
                'injected_file_content_bytes': -1,
                'injected_file_path_bytes': -1,
                'injected_files': -1,
                'instances': -1,
                'key_pairs': -1,
                'metadata_items': -1,
                'ram': -1,
                'security_group_rules': -1,
                'security_groups': -1,
                'server_group_members': -1,
                'server_groups': -1,
            }
        }
        self.assertEqual(expected_response, response)
        mock_create.assert_called_once_with(req.environ['nova.context'],
                                            uuids.project_id, "server_groups",
                                            2, user_id=None)

    @mock.patch.object(objects.Quotas, "create_limit")
    def test_update_v21_user(self, mock_create):
        req = fakes.HTTPRequest.blank("?user_id=42")
        body = {'quota_set': {'key_pairs': 52}}
        response = self.controller.update(req, uuids.project_id, body=body)
        expected_response = {
            'quota_set': {
                'cores': -1,
                'fixed_ips': -1,
                'floating_ips': -1,
                'injected_file_content_bytes': -1,
                'injected_file_path_bytes': -1,
                'injected_files': -1,
                'instances': -1,
                'key_pairs': -1,
                'metadata_items': -1,
                'ram': -1,
                'security_group_rules': -1,
                'security_groups': -1,
                'server_group_members': -1,
                'server_groups': -1,
            }
        }
        self.assertEqual(expected_response, response)
        mock_create.assert_called_once_with(req.environ['nova.context'],
                                            uuids.project_id, "key_pairs", 52,
                                            user_id="42")

    def test_defaults_v21(self):
        req = fakes.HTTPRequest.blank("")
        response = self.controller.defaults(req, uuids.project_id)
        expected_response = {
            'quota_set': {
                'id': uuids.project_id,
                'cores': -1,
                'fixed_ips': -1,
                'floating_ips': -1,
                'injected_file_content_bytes': -1,
                'injected_file_path_bytes': -1,
                'injected_files': -1,
                'instances': -1,
                'key_pairs': -1,
                'metadata_items': -1,
                'ram': -1,
                'security_group_rules': -1,
                'security_groups': -1,
                'server_group_members': -1,
                'server_groups': -1,
            }
        }
        self.assertEqual(expected_response, response)

    @mock.patch('nova.objects.Quotas.destroy_all_by_project')
    def test_quotas_delete(self, mock_destroy_all_by_project):
        req = fakes.HTTPRequest.blank("")
        self.controller.delete(req, "1234")
        mock_destroy_all_by_project.assert_called_once_with(
            req.environ['nova.context'], "1234")

    @mock.patch('nova.objects.Quotas.destroy_all_by_project_and_user')
    def test_user_quotas_delete(self, mock_destroy_all_by_user):
        req = fakes.HTTPRequest.blank("?user_id=42")
        self.controller.delete(req, "1234")
        mock_destroy_all_by_user.assert_called_once_with(
            req.environ['nova.context'], "1234", "42")


class UnifiedLimitsQuotaSetsTest(NoopQuotaSetsTest):
    quota_driver = "nova.quota.UnifiedLimitsDriver"
    # this matches what the db driver returns
    expected_detail = {'in_use': 0, 'limit': -1, 'reserved': 0}

    def setUp(self):
        super(UnifiedLimitsQuotaSetsTest, self).setUp()
        reglimits = {local_limit.SERVER_METADATA_ITEMS: 128,
                     local_limit.INJECTED_FILES: 5,
                     local_limit.INJECTED_FILES_CONTENT: 10 * 1024,
                     local_limit.INJECTED_FILES_PATH: 255,
                     local_limit.KEY_PAIRS: 100,
                     local_limit.SERVER_GROUPS: 12,
                     local_limit.SERVER_GROUP_MEMBERS: 10}
        self.limit_fixture = self.useFixture(
            limit_fixture.LimitFixture(reglimits, {}))

    @mock.patch.object(placement_limit, "get_legacy_project_limits")
    def test_show_v21(self, mock_proj):
        mock_proj.return_value = {"instances": 1, "cores": 2, "ram": 3}
        req = fakes.HTTPRequest.blank("")
        response = self.controller.show(req, uuids.project_id)
        expected_response = {
            'quota_set': {
                'id': uuids.project_id,
                'cores': 2,
                'fixed_ips': -1,
                'floating_ips': -1,
                'injected_file_content_bytes': 10240,
                'injected_file_path_bytes': 255,
                'injected_files': 5,
                'instances': 1,
                'key_pairs': 100,
                'metadata_items': 128,
                'ram': 3,
                'security_group_rules': -1,
                'security_groups': -1,
                'server_group_members': 10,
                'server_groups': 12,
            }
        }
        self.assertEqual(expected_response, response)

    @mock.patch.object(placement_limit, "get_legacy_project_limits")
    def test_show_v257(self, mock_proj):
        mock_proj.return_value = {"instances": 1, "cores": 2, "ram": 3}
        req = fakes.HTTPRequest.blank("", version='2.57')
        response = self.controller.show(req, uuids.project_id)
        expected_response = {
            'quota_set': {
                'id': uuids.project_id,
                'cores': 2,
                'instances': 1,
                'key_pairs': 100,
                'metadata_items': 128,
                'ram': 3,
                'server_group_members': 10,
                'server_groups': 12}}
        self.assertEqual(expected_response, response)

    @mock.patch.object(placement_limit, "get_legacy_counts")
    @mock.patch.object(placement_limit, "get_legacy_project_limits")
    @mock.patch.object(objects.InstanceGroupList, "get_counts")
    def test_detail_v21(self, mock_count, mock_proj, mock_kcount):
        mock_proj.return_value = {"instances": 1, "cores": 2, "ram": 3}
        mock_kcount.return_value = {"instances": 4, "cores": 5, "ram": 6}
        mock_count.return_value = {'project': {'server_groups': 9}}
        req = fakes.HTTPRequest.blank("")
        response = self.controller.detail(req, uuids.project_id)
        expected_response = {
            'quota_set': {
                'id': uuids.project_id,
                'cores': {
                    'in_use': 5, 'limit': 2, 'reserved': 0},
                'fixed_ips': self.expected_detail,
                'floating_ips': self.expected_detail,
                'injected_file_content_bytes': {
                    'in_use': 0, 'limit': 10240, 'reserved': 0},
                'injected_file_path_bytes': {
                    'in_use': 0, 'limit': 255, 'reserved': 0},
                'injected_files': {
                    'in_use': 0, 'limit': 5, 'reserved': 0},
                'instances': {
                    'in_use': 4, 'limit': 1, 'reserved': 0},
                'key_pairs': {
                    'in_use': 0, 'limit': 100, 'reserved': 0},
                'metadata_items': {
                    'in_use': 0, 'limit': 128, 'reserved': 0},
                'ram': {
                    'in_use': 6, 'limit': 3, 'reserved': 0},
                'security_group_rules': self.expected_detail,
                'security_groups': self.expected_detail,
                'server_group_members': {
                    'in_use': 0, 'limit': 10, 'reserved': 0},
                'server_groups': {
                    'in_use': 9, 'limit': 12, 'reserved': 0},
            }
        }
        self.assertEqual(expected_response, response)

    @mock.patch.object(placement_limit, "get_legacy_counts")
    @mock.patch.object(placement_limit, "get_legacy_project_limits")
    @mock.patch.object(objects.InstanceGroupList, "get_counts")
    def test_detail_v21_user(self, mock_count, mock_proj, mock_kcount):
        mock_proj.return_value = {"instances": 1, "cores": 2, "ram": 3}
        mock_kcount.return_value = {"instances": 4, "cores": 5, "ram": 6}
        mock_count.return_value = {'project': {'server_groups': 9}}
        req = fakes.HTTPRequest.blank("?user_id=42")
        response = self.controller.detail(req, uuids.project_id)
        expected_response = {
            'quota_set': {
                'id': uuids.project_id,
                'cores': {
                    'in_use': 5, 'limit': 2, 'reserved': 0},
                'fixed_ips': self.expected_detail,
                'floating_ips': self.expected_detail,
                'injected_file_content_bytes': {
                    'in_use': 0, 'limit': 10240, 'reserved': 0},
                'injected_file_path_bytes': {
                    'in_use': 0, 'limit': 255, 'reserved': 0},
                'injected_files': {
                    'in_use': 0, 'limit': 5, 'reserved': 0},
                'instances': {
                    'in_use': 4, 'limit': 1, 'reserved': 0},
                'key_pairs': {
                    'in_use': 0, 'limit': 100, 'reserved': 0},
                'metadata_items': {
                    'in_use': 0, 'limit': 128, 'reserved': 0},
                'ram': {
                    'in_use': 6, 'limit': 3, 'reserved': 0},
                'security_group_rules': self.expected_detail,
                'security_groups': self.expected_detail,
                'server_group_members': {
                    'in_use': 0, 'limit': 10, 'reserved': 0},
                'server_groups': {
                    'in_use': 9, 'limit': 12, 'reserved': 0},
            }
        }
        self.assertEqual(expected_response, response)

    @mock.patch.object(placement_limit, "get_legacy_project_limits")
    @mock.patch.object(objects.Quotas, "create_limit")
    def test_update_v21(self, mock_create, mock_proj):
        mock_proj.return_value = {"instances": 1, "cores": 2, "ram": 3}
        req = fakes.HTTPRequest.blank("")
        # TODO(johngarbutt) still need to implement get_settable_quotas
        body = {'quota_set': {'server_groups': 2}}
        response = self.controller.update(req, uuids.project_id, body=body)
        expected_response = {
            'quota_set': {
                'cores': 2,
                'fixed_ips': -1,
                'floating_ips': -1,
                'injected_file_content_bytes': 10240,
                'injected_file_path_bytes': 255,
                'injected_files': 5,
                'instances': 1,
                'key_pairs': 100,
                'metadata_items': 128,
                'ram': 3,
                'security_group_rules': -1,
                'security_groups': -1,
                'server_group_members': 10,
                'server_groups': 12,
            }
        }
        self.assertEqual(expected_response, response)
        self.assertEqual(0, mock_create.call_count)

    @mock.patch.object(placement_limit, "get_legacy_project_limits")
    @mock.patch.object(objects.Quotas, "create_limit")
    def test_update_v21_user(self, mock_create, mock_proj):
        mock_proj.return_value = {"instances": 1, "cores": 2, "ram": 3}
        req = fakes.HTTPRequest.blank("?user_id=42")
        body = {'quota_set': {'key_pairs': 52}}
        response = self.controller.update(req, uuids.project_id, body=body)
        expected_response = {
            'quota_set': {
                'cores': 2,
                'fixed_ips': -1,
                'floating_ips': -1,
                'injected_file_content_bytes': 10240,
                'injected_file_path_bytes': 255,
                'injected_files': 5,
                'instances': 1,
                'key_pairs': 100,
                'metadata_items': 128,
                'ram': 3,
                'security_group_rules': -1,
                'security_groups': -1,
                'server_group_members': 10,
                'server_groups': 12,
            }
        }
        self.assertEqual(expected_response, response)
        self.assertEqual(0, mock_create.call_count)

    @mock.patch.object(placement_limit, "get_legacy_default_limits")
    def test_defaults_v21(self, mock_default):
        mock_default.return_value = {"instances": 1, "cores": 2, "ram": 3}
        req = fakes.HTTPRequest.blank("")
        response = self.controller.defaults(req, uuids.project_id)
        expected_response = {
            'quota_set': {
                'id': uuids.project_id,
                'cores': 2,
                'fixed_ips': -1,
                'floating_ips': -1,
                'injected_file_content_bytes': 10240,
                'injected_file_path_bytes': 255,
                'injected_files': 5,
                'instances': 1,
                'key_pairs': 100,
                'metadata_items': 128,
                'ram': 3,
                'security_group_rules': -1,
                'security_groups': -1,
                'server_group_members': 10,
                'server_groups': 12,
            }
        }
        self.assertEqual(expected_response, response)

    def test_defaults_v21_different_limit_values(self):
        reglimits = {local_limit.SERVER_METADATA_ITEMS: 7,
                     local_limit.INJECTED_FILES: 6,
                     local_limit.INJECTED_FILES_CONTENT: 4,
                     local_limit.INJECTED_FILES_PATH: 5,
                     local_limit.KEY_PAIRS: 1,
                     local_limit.SERVER_GROUPS: 3,
                     local_limit.SERVER_GROUP_MEMBERS: 2}
        self.limit_fixture.reglimits = reglimits

        req = fakes.HTTPRequest.blank("")
        response = self.controller.defaults(req, uuids.project_id)
        expected_response = {
            'quota_set': {
                'id': uuids.project_id,
                'cores': 0,
                'fixed_ips': -1,
                'floating_ips': -1,
                'injected_file_content_bytes': 4,
                'injected_file_path_bytes': 5,
                'injected_files': 6,
                'instances': 0,
                'key_pairs': 1,
                'metadata_items': 7,
                'ram': 0,
                'security_group_rules': -1,
                'security_groups': -1,
                'server_group_members': 2,
                'server_groups': 3,
            }
        }
        self.assertEqual(expected_response, response)

    @mock.patch('nova.objects.Quotas.destroy_all_by_project')
    def test_quotas_delete(self, mock_destroy_all_by_project):
        req = fakes.HTTPRequest.blank("")
        self.controller.delete(req, "1234")
        # Ensure destroy isn't called for unified limits
        self.assertEqual(0, mock_destroy_all_by_project.call_count)

    @mock.patch('nova.objects.Quotas.destroy_all_by_project_and_user')
    def test_user_quotas_delete(self, mock_destroy_all_by_user):
        req = fakes.HTTPRequest.blank("?user_id=42")
        self.controller.delete(req, "1234")
        # Ensure destroy isn't called for unified limits
        self.assertEqual(0, mock_destroy_all_by_user.call_count)



# Copyright 2013 OpenStack Foundation
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from unittest import mock

from oslo_utils.fixture import uuidsentinel as uuids

from nova import block_device
from nova.compute import vm_states
import nova.conf
from nova import context
from nova import objects
from nova.objects import block_device as block_device_obj
from nova import test
from nova.tests.unit import fake_instance
from nova.virt import imagecache

CONF = nova.conf.CONF

swap_bdm_128 = [block_device.BlockDeviceDict(
        {'id': 1, 'instance_uuid': uuids.instance,
         'device_name': '/dev/sdb1',
         'source_type': 'blank',
         'destination_type': 'local',
         'delete_on_termination': True,
         'guest_format': 'swap',
         'disk_bus': 'scsi',
         'volume_size': 128,
         'boot_index': -1})]

swap_bdm_256 = [block_device.BlockDeviceDict(
        {'id': 1, 'instance_uuid': uuids.instance,
         'device_name': '/dev/sdb1',
         'source_type': 'blank',
         'destination_type': 'local',
         'delete_on_termination': True,
         'guest_format': 'swap',
         'disk_bus': 'scsi',
         'volume_size': 256,
         'boot_index': -1})]

ephemeral_bdm = [block_device.BlockDeviceDict(
        {'id': 1, 'instance_uuid': uuids.instance,
         'device_name': '/dev/sdc1',
         'source_type': 'blank',
         'destination_type': 'local',
         'disk_bus': 'scsi',
         'device_type': 'disk',
         'volume_size': 4,
         'guest_format': 'ext4',
         'delete_on_termination': True,
         'boot_index': -1})]


class ImageCacheManagerTests(test.NoDBTestCase):

    def test_configuration_defaults(self):
        self.assertEqual(2400, CONF.image_cache.manager_interval)
        self.assertEqual('_base', CONF.image_cache.subdirectory_name)
        self.assertTrue(CONF.image_cache.remove_unused_base_images)
        self.assertEqual(
            24 * 3600,
            CONF.image_cache.remove_unused_original_minimum_age_seconds)

    def test_cache_manager(self):
        cache_manager = imagecache.ImageCacheManager()
        self.assertTrue(cache_manager.remove_unused_base_images)
        self.assertRaises(NotImplementedError,
                          cache_manager.update, None, [])
        self.assertRaises(NotImplementedError,
                          cache_manager._get_base)
        self.assertRaises(NotImplementedError,
                          cache_manager._scan_base_images, None)
        self.assertRaises(NotImplementedError,
                          cache_manager._age_and_verify_cached_images,
                          None, [], None)

    @mock.patch.object(objects.BlockDeviceMappingList,
                       'bdms_by_instance_uuid')
    def test_list_running_instances(self, mock_bdms_by_uuid):
        instances = [{'image_ref': '1',
                      'host': CONF.host,
                      'id': '1',
                      'uuid': uuids.instance_1,
                      'vm_state': '',
                      'task_state': ''},
                     {'image_ref': '2',
                      'host': CONF.host,
                      'id': '2',
                      'uuid': uuids.instance_2,
                      'vm_state': '',
                      'task_state': ''},
                     {'image_ref': '2',
                      'kernel_id': '21',
                      'ramdisk_id': '22',
                      'host': 'remotehost',
                      'id': '3',
                      'uuid': uuids.instance_3,
                      'vm_state': '',
                      'task_state': ''}]

        all_instances = [fake_instance.fake_instance_obj(None, **instance)
                         for instance in instances]

        image_cache_manager = imagecache.ImageCacheManager()

        ctxt = context.get_admin_context()
        swap_bdm_256_list = block_device_obj.block_device_make_list_from_dicts(
            ctxt, swap_bdm_256 + ephemeral_bdm)
        swap_bdm_128_list = block_device_obj.block_device_make_list_from_dicts(
            ctxt, swap_bdm_128)
        mock_bdms_by_uuid.return_value = {uuids.instance_1: swap_bdm_256_list,
                                          uuids.instance_2: swap_bdm_128_list,
                                          uuids.instance_3: swap_bdm_128_list}
        # The argument here should be a context, but it's mocked out
        running = image_cache_manager._list_running_instances(ctxt,
            all_instances)

        mock_bdms_by_uuid.assert_called_once_with(ctxt,
                     [uuids.instance_1, uuids.instance_2, uuids.instance_3])
        self.assertEqual(4, len(running['used_images']))
        self.assertEqual((1, 0, ['instance-00000001']),
                         running['used_images']['1'])
        self.assertEqual((1, 1, ['instance-00000002',
                                 'instance-00000003']),
                         running['used_images']['2'])
        self.assertEqual((0, 1, ['instance-00000003']),
                         running['used_images']['21'])
        self.assertEqual((0, 1, ['instance-00000003']),
                         running['used_images']['22'])

        self.assertIn('instance-00000001', running['instance_names'])
        self.assertIn(uuids.instance_1, running['instance_names'])

        self.assertEqual(len(running['used_swap_images']), 2)
        self.assertIn('swap_128', running['used_swap_images'])
        self.assertIn('swap_256', running['used_swap_images'])

        self.assertEqual(len(running['used_ephemeral_images']), 1)
        self.assertIn('ephemeral_4_0706d66', running['used_ephemeral_images'])

    @mock.patch.object(objects.BlockDeviceMappingList,
                       'bdms_by_instance_uuid')
    def test_list_resizing_instances(self, mock_bdms_by_uuid):
        instances = [{'image_ref': '1',
                      'host': CONF.host,
                      'id': '1',
                      'uuid': uuids.instance,
                      'vm_state': vm_states.RESIZED,
                      'task_state': None}]

        all_instances = [fake_instance.fake_instance_obj(None, **instance)
                         for instance in instances]

        image_cache_manager = imagecache.ImageCacheManager()

        ctxt = context.get_admin_context()
        bdms = block_device_obj.block_device_make_list_from_dicts(
            ctxt, swap_bdm_256)
        mock_bdms_by_uuid.return_value = {uuids.instance: bdms}

        running = image_cache_manager._list_running_instances(ctxt,
            all_instances)

        mock_bdms_by_uuid.assert_called_once_with(ctxt, [uuids.instance])
        self.assertEqual(1, len(running['used_images']))
        self.assertEqual((1, 0, ['instance-00000001']),
                         running['used_images']['1'])
        self.assertEqual(set(['instance-00000001', uuids.instance,
                              'instance-00000001_resize',
                              '%s_resize' % uuids.instance]),
                         running['instance_names'])



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import nova

from nova import test
from nova.tests import fixtures as nova_fixtures
from nova.tests.functional import fixtures as func_fixtures
from nova.tests.functional import integrated_helpers
import oslo_messaging as messaging
from unittest import mock


class PostLiveMigrationFail(
    test.TestCase, integrated_helpers.InstanceHelperMixin):
    """Regression test for bug 2143972
    """

    def setUp(self):
        super().setUp()
        self.useFixture(nova_fixtures.NeutronFixture(self))
        self.glance = self.useFixture(nova_fixtures.GlanceFixture(self))
        self.useFixture(func_fixtures.PlacementFixture())
        self.useFixture(nova_fixtures.HostNameWeigherFixture())

        self.start_service('conductor')
        self.start_service('scheduler')

        api_fixture = self.useFixture(nova_fixtures.OSAPIFixture(
            api_version='v2.1'))

        self.api = api_fixture.admin_api
        self.api.microversion = 'latest'

        self.src = self._start_compute(host='host1')
        self.dest = self._start_compute(host='host2')

    def test_post_live_migration_keystone_fails(self):
        """Test for bug 2143972.
        This test simulate a service failure during post live migration.
        """
        orig_function = nova.network.neutron.API.setup_networks_on_host
        service_exception = messaging.RemoteError(
            exc_type="Remote Error",
            value="ServiceUnavailable The server is currently "
            "unavailable.")
        self.count_call = 0

        # Fist setup_networks_on_host call is in the pre_live_migration method.
        # Second setup_networks_on_host call is in the post_live_migration
        # method.
        def fake_setup_networks_on_host(*args, **kwargs):
            """Call the original function first and then raise an exception
            simulating keystone or neutron not available.
            """
            self.count_call += 1
            if self.count_call > 1:
                raise service_exception
            return orig_function(*args, **kwargs)

        server = self._create_server(networks=[])
        self.assertEqual(self.src.host, server['OS-EXT-SRV-ATTR:host'])

        with mock.patch(
            'nova.network.neutron.API.setup_networks_on_host',
            side_effect=fake_setup_networks_on_host
        ):

            # FIXME(Uggla) A failure with post_live_migration_at_destination
            # leave the instance in MIGRATING status and
            # migration_expected_state to completed because the exception
            # is not raised by the compute manager.  This is mainly due to
            # the previous behavior that would not want to break, allowing a
            # migration cleanup.  A recent patch allows better handling of post
            # migration errors, so we can now raise an exception in such cases.
            server = self._live_migrate(
                server, migration_expected_state='completed',
                server_expected_state='MIGRATING')

            self.assertEqual(self.src.host, server['OS-EXT-SRV-ATTR:host'])



#    Copyright 2014 Red Hat, Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import textwrap
from unittest import mock

import pycodestyle

from nova.hacking import checks
from nova import test


class HackingTestCase(test.NoDBTestCase):
    """This class tests the hacking checks in nova.hacking.checks by passing
    strings to the check methods like the pycodestyle/flake8 parser would. The
    parser loops over each line in the file and then passes the parameters to
    the check method. The parameter names in the check method dictate what type
    of object is passed to the check method. The parameter types are::

        logical_line: A processed line with the following modifications:
            - Multi-line statements converted to a single line.
            - Stripped left and right.
            - Contents of strings replaced with "xxx" of same length.
            - Comments removed.
        physical_line: Raw line of text from the input file.
        lines: a list of the raw lines from the input file
        tokens: the tokens that contribute to this logical line
        line_number: line number in the input file
        total_lines: number of lines in the input file
        blank_lines: blank lines before this one
        indent_char: indentation character in this file (" " or "\t")
        indent_level: indentation (with tabs expanded to multiples of 8)
        previous_indent_level: indentation on previous line
        previous_logical: previous logical line
        filename: Path of the file being run through pycodestyle

    When running a test on a check method the return will be False/None if
    there is no violation in the sample input. If there is an error a tuple is
    returned with a position in the line, and a message. So to check the result
    just assertTrue if the check is expected to fail and assertFalse if it
    should pass.
    """

    def test_virt_driver_imports(self):

        expect = (0, "N311: importing code from other virt drivers forbidden")

        self.assertEqual(expect, checks.import_no_virt_driver_import_deps(
            "from nova.virt.libvirt import utils as libvirt_utils",
            "./nova/virt/zvm/driver.py"))

        self.assertEqual(expect, checks.import_no_virt_driver_import_deps(
            "import nova.virt.libvirt.utils as libvirt_utils",
            "./nova/virt/zvm/driver.py"))

        self.assertIsNone(checks.import_no_virt_driver_import_deps(
            "from nova.virt.libvirt import utils as libvirt_utils",
            "./nova/virt/libvirt/driver.py"))

    def test_virt_driver_config_vars(self):
        self.assertIsInstance(checks.import_no_virt_driver_config_deps(
            "CONF.import_opt('volume_drivers', "
            "'nova.virt.libvirt.driver', group='libvirt')",
            "./nova/virt/zm/driver.py"), tuple)

        self.assertIsNone(checks.import_no_virt_driver_config_deps(
            "CONF.import_opt('volume_drivers', "
            "'nova.virt.libvirt.driver', group='libvirt')",
            "./nova/virt/libvirt/volume.py"))

    def test_assert_true_instance(self):
        self.assertEqual(len(list(checks.assert_true_instance(
            "self.assertTrue(isinstance(e, "
            "exception.BuildAbortException))"))), 1)

        self.assertEqual(
            len(list(checks.assert_true_instance("self.assertTrue()"))), 0)

    def test_assert_equal_type(self):
        self.assertEqual(len(list(checks.assert_equal_type(
            "self.assertEqual(type(als['QuicAssist']), list)"))), 1)

        self.assertEqual(
            len(list(checks.assert_equal_type("self.assertTrue()"))), 0)

    def test_assert_equal_in(self):
        self.assertEqual(len(list(checks.assert_equal_in(
            "self.assertEqual(a in b, True)"))), 1)

        self.assertEqual(len(list(checks.assert_equal_in(
            "self.assertEqual('str' in 'string', True)"))), 1)

        self.assertEqual(len(list(checks.assert_equal_in(
            "self.assertEqual(any(a==1 for a in b), True)"))), 0)

        self.assertEqual(len(list(checks.assert_equal_in(
            "self.assertEqual(True, a in b)"))), 1)

        self.assertEqual(len(list(checks.assert_equal_in(
            "self.assertEqual(True, 'str' in 'string')"))), 1)

        self.assertEqual(len(list(checks.assert_equal_in(
            "self.assertEqual(True, any(a==1 for a in b))"))), 0)

        self.assertEqual(len(list(checks.assert_equal_in(
            "self.assertEqual(a in b, False)"))), 1)

        self.assertEqual(len(list(checks.assert_equal_in(
            "self.assertEqual('str' in 'string', False)"))), 1)

        self.assertEqual(len(list(checks.assert_equal_in(
            "self.assertEqual(any(a==1 for a in b), False)"))), 0)

        self.assertEqual(len(list(checks.assert_equal_in(
            "self.assertEqual(False, a in b)"))), 1)

        self.assertEqual(len(list(checks.assert_equal_in(
            "self.assertEqual(False, 'str' in 'string')"))), 1)

        self.assertEqual(len(list(checks.assert_equal_in(
            "self.assertEqual(False, any(a==1 for a in b))"))), 0)

    def test_assert_true_or_false_with_in_or_not_in(self):
        self.assertEqual(len(list(checks.assert_true_or_false_with_in(
            "self.assertTrue(A in B)"))), 1)

        self.assertEqual(len(list(checks.assert_true_or_false_with_in(
            "self.assertFalse(A in B)"))), 1)

        self.assertEqual(len(list(checks.assert_true_or_false_with_in(
            "self.assertTrue(A not in B)"))), 1)

        self.assertEqual(len(list(checks.assert_true_or_false_with_in(
            "self.assertFalse(A not in B)"))), 1)

        self.assertEqual(len(list(checks.assert_true_or_false_with_in(
            "self.assertTrue(A in B, 'some message')"))), 1)

        self.assertEqual(len(list(checks.assert_true_or_false_with_in(
            "self.assertFalse(A in B, 'some message')"))), 1)

        self.assertEqual(len(list(checks.assert_true_or_false_with_in(
            "self.assertTrue(A not in B, 'some message')"))), 1)

        self.assertEqual(len(list(checks.assert_true_or_false_with_in(
            "self.assertFalse(A not in B, 'some message')"))), 1)

        self.assertEqual(len(list(checks.assert_true_or_false_with_in(
            "self.assertTrue(A in 'some string with spaces')"))), 1)

        self.assertEqual(len(list(checks.assert_true_or_false_with_in(
            "self.assertTrue(A in ['1', '2', '3'])"))), 1)

        self.assertEqual(len(list(checks.assert_true_or_false_with_in(
            "self.assertTrue(A in [1, 2, 3])"))), 1)

        self.assertEqual(len(list(checks.assert_true_or_false_with_in(
            "self.assertTrue(any(A > 5 for A in B))"))), 0)

        self.assertEqual(len(list(checks.assert_true_or_false_with_in(
            "self.assertTrue(any(A > 5 for A in B), 'some message')"))), 0)

        self.assertEqual(len(list(checks.assert_true_or_false_with_in(
            "self.assertFalse(some in list1 and some2 in list2)"))), 0)

    def test_no_translate_logs(self):
        self.assertEqual(len(list(checks.no_translate_logs(
            "LOG.debug(_('foo'))", "nova/scheduler/foo.py"))), 1)

        self.assertEqual(len(list(checks.no_translate_logs(
            "LOG.debug('foo')", "nova/scheduler/foo.py"))), 0)

        self.assertEqual(len(list(checks.no_translate_logs(
            "LOG.info(_('foo'))", "nova/scheduler/foo.py"))), 1)

    def test_no_setting_conf_directly_in_tests(self):
        self.assertEqual(len(list(checks.no_setting_conf_directly_in_tests(
            "CONF.option = 1", "nova/tests/test_foo.py"))), 1)

        self.assertEqual(len(list(checks.no_setting_conf_directly_in_tests(
            "CONF.group.option = 1", "nova/tests/test_foo.py"))), 1)

        self.assertEqual(len(list(checks.no_setting_conf_directly_in_tests(
            "CONF.option = foo = 1", "nova/tests/test_foo.py"))), 1)

        # Shouldn't fail with comparisons
        self.assertEqual(len(list(checks.no_setting_conf_directly_in_tests(
            "CONF.option == 'foo'", "nova/tests/test_foo.py"))), 0)

        self.assertEqual(len(list(checks.no_setting_conf_directly_in_tests(
            "CONF.option != 1", "nova/tests/test_foo.py"))), 0)

        # Shouldn't fail since not in nova/tests/
        self.assertEqual(len(list(checks.no_setting_conf_directly_in_tests(
            "CONF.option = 1", "nova/compute/foo.py"))), 0)

    def test_no_mutable_default_args(self):
        self.assertEqual(1, len(list(checks.no_mutable_default_args(
            "def get_info_from_bdm(virt_type, bdm, mapping=[])"))))

        self.assertEqual(0, len(list(checks.no_mutable_default_args(
            "defined = []"))))

        self.assertEqual(0, len(list(checks.no_mutable_default_args(
            "defined, undefined = [], {}"))))

    def test_check_explicit_underscore_import(self):
        self.assertEqual(len(list(checks.check_explicit_underscore_import(
            "msg = _('My message')",
            "cinder/tests/other_files.py"))), 1)
        self.assertEqual(len(list(checks.check_explicit_underscore_import(
            "from cinder.i18n import _",
            "cinder/tests/other_files.py"))), 0)
        self.assertEqual(len(list(checks.check_explicit_underscore_import(
            "msg = _('My message')",
            "cinder/tests/other_files.py"))), 0)
        self.assertEqual(len(list(checks.check_explicit_underscore_import(
            "from cinder.i18n import _",
            "cinder/tests/other_files2.py"))), 0)
        self.assertEqual(len(list(checks.check_explicit_underscore_import(
            "msg = _('My message')",
            "cinder/tests/other_files2.py"))), 0)
        self.assertEqual(len(list(checks.check_explicit_underscore_import(
            "_ = translations.ugettext",
            "cinder/tests/other_files3.py"))), 0)
        self.assertEqual(len(list(checks.check_explicit_underscore_import(
            "msg = _('My message')",
            "cinder/tests/other_files3.py"))), 0)

    def test_use_jsonutils(self):
        def __get_msg(fun):
            msg = ("N324: jsonutils.%(fun)s must be used instead of "
                   "json.%(fun)s" % {'fun': fun})
            return [(0, msg)]

        for method in ('dump', 'dumps', 'load', 'loads'):
            self.assertEqual(
                __get_msg(method),
                list(checks.use_jsonutils(
                    "json.%s(" % method, "./nova/virt/libvirt/driver.py")),
            )
            self.assertEqual(
                0,
                len(list(checks.use_jsonutils(
                    "jsonx.%s(" % method, "./nova/virt/libvirt/driver.py"))),
            )

        self.assertEqual(
            0,
            len(list(checks.use_jsonutils(
                "json.dumb", "./nova/virt/libvirt/driver.py"))),
        )

    # We are patching pycodestyle so that only the check under test is actually
    # installed.
    @mock.patch('pycodestyle._checks',
                {'physical_line': {}, 'logical_line': {}, 'tree': {}})
    def _run_check(self, code, checker, filename=None):
        pycodestyle.register_check(checker)

        lines = textwrap.dedent(code).lstrip().splitlines(True)

        checker = pycodestyle.Checker(filename=filename, lines=lines)
        # NOTE(sdague): the standard reporter has printing to stdout
        # as a normal part of check_all, which bleeds through to the
        # test output stream in an unhelpful way. This blocks that printing.
        with mock.patch('pycodestyle.StandardReport.get_file_results'):
            checker.check_all()
        checker.report._deferred_print.sort()
        return checker.report._deferred_print

    def _assert_has_errors(self, code, checker, expected_errors=None,
                           filename=None):
        actual_errors = [e[:3] for e in
                         self._run_check(code, checker, filename)]
        self.assertEqual(expected_errors or [], actual_errors)

    def _assert_has_no_errors(self, code, checker, filename=None):
        self._assert_has_errors(code, checker, filename=filename)

    def test_api_version_decorator_check(self):
        code = """
               @some_other_decorator
               @wsgi.api_version("2.5")
               def my_method():
                   pass
               """
        self._assert_has_errors(code, checks.check_api_version_decorator,
                                expected_errors=[(2, 0, "N332")])

    def test_oslo_assert_raises_regexp(self):
        code = """
               self.assertRaisesRegexp(ValueError,
                                       "invalid literal for.*XYZ'$",
                                       int,
                                       'XYZ')
               """
        self._assert_has_errors(code, checks.assert_raises_regexp,
                                expected_errors=[(1, 0, "N335")])

    def test_api_version_decorator_check_no_errors(self):
        codelist = [
            """
            class ControllerClass():
                @wsgi.api_version("2.5")
                def my_method():
                    pass
            """,
            """
            @some_other_decorator
            @mock.patch('foo', return_value=api_versions.APIVersion("2.5"))
            def my_method():
                pass
            """]
        for code in codelist:
            self._assert_has_no_errors(
                code, checks.check_api_version_decorator)

    def test_trans_add(self):

        checker = checks.CheckForTransAdd
        code = """
               def fake_tran(msg):
                   return msg


               _ = fake_tran


               def f(a, b):
                   msg = _('test') + 'add me'
                   msg = 'add to me' + _('test')
                   return msg
               """
        errors = [(9, 10, 'N326'), (10, 24, 'N326')]
        self._assert_has_errors(code, checker, expected_errors=errors)

        code = """
               def f(a, b):
                   msg = 'test' + 'add me'
                   return msg
               """
        self._assert_has_no_errors(code, checker)

    def test_dict_constructor_with_list_copy(self):
        self.assertEqual(1, len(list(checks.dict_constructor_with_list_copy(
            "    dict([(i, connect_info[i])"))))

        self.assertEqual(1, len(list(checks.dict_constructor_with_list_copy(
            "    attrs = dict([(k, _from_json(v))"))))

        self.assertEqual(1, len(list(checks.dict_constructor_with_list_copy(
            "        type_names = dict((value, key) for key, value in"))))

        self.assertEqual(1, len(list(checks.dict_constructor_with_list_copy(
            "   dict((value, key) for key, value in"))))

        self.assertEqual(1, len(list(checks.dict_constructor_with_list_copy(
            "foo(param=dict((k, v) for k, v in bar.items()))"))))

        self.assertEqual(1, len(list(checks.dict_constructor_with_list_copy(
            " dict([[i,i] for i in range(3)])"))))

        self.assertEqual(1, len(list(checks.dict_constructor_with_list_copy(
            "  dd = dict([i,i] for i in range(3))"))))

        self.assertEqual(0, len(list(checks.dict_constructor_with_list_copy(
            "        create_kwargs = dict(snapshot=snapshot,"))))

        self.assertEqual(0, len(list(checks.dict_constructor_with_list_copy(
            "      self._render_dict(xml, data_el, data.__dict__)"))))

    def test_check_http_not_implemented(self):
        code = """
               except NotImplementedError:
                   common.raise_http_not_implemented_error()
               """
        filename = "nova/api/openstack/compute/v21/test.py"
        self._assert_has_no_errors(code, checks.check_http_not_implemented,
                                   filename=filename)

        code = """
               except NotImplementedError:
                   msg = _("Unable to set password on instance")
                   raise exc.HTTPNotImplemented(explanation=msg)
               """
        errors = [(3, 4, 'N339')]
        self._assert_has_errors(code, checks.check_http_not_implemented,
                                expected_errors=errors, filename=filename)

    def test_check_contextlib_use(self):
        code = """
               with test.nested(
                   mock.patch.object(network_model.NetworkInfo, 'hydrate'),
                   mock.patch.object(objects.InstanceInfoCache, 'save'),
               ) as (
                   hydrate_mock, save_mock
               )
               """
        filename = "nova/api/openstack/compute/v21/test.py"
        self._assert_has_no_errors(code, checks.check_no_contextlib_nested,
                                   filename=filename)
        code = """
               with contextlib.nested(
                   mock.patch.object(network_model.NetworkInfo, 'hydrate'),
                   mock.patch.object(objects.InstanceInfoCache, 'save'),
               ) as (
                   hydrate_mock, save_mock
               )
               """
        filename = "nova/api/openstack/compute/legacy_v2/test.py"
        errors = [(1, 0, 'N341')]
        self._assert_has_errors(code, checks.check_no_contextlib_nested,
                                expected_errors=errors, filename=filename)

    def test_check_greenthread_spawns(self):
        errors = [(1, 0, "N340")]

        code = "greenthread.spawn(func, arg1, kwarg1=kwarg1)"
        self._assert_has_errors(code, checks.check_greenthread_spawns,
                                expected_errors=errors)

        code = "greenthread.spawn_n(func, arg1, kwarg1=kwarg1)"
        self._assert_has_errors(code, checks.check_greenthread_spawns,
                                expected_errors=errors)

        code = "eventlet.greenthread.spawn(func, arg1, kwarg1=kwarg1)"
        self._assert_has_errors(code, checks.check_greenthread_spawns,
                                expected_errors=errors)

        code = "eventlet.spawn(func, arg1, kwarg1=kwarg1)"
        self._assert_has_errors(code, checks.check_greenthread_spawns,
                                expected_errors=errors)

        code = "eventlet.spawn_n(func, arg1, kwarg1=kwarg1)"
        self._assert_has_errors(code, checks.check_greenthread_spawns,
                                expected_errors=errors)

        code = "nova.utils.spawn(func, arg1, kwarg1=kwarg1)"
        self._assert_has_no_errors(code, checks.check_greenthread_spawns)

        code = "nova.utils.spawn_n(func, arg1, kwarg1=kwarg1)"
        self._assert_has_no_errors(code, checks.check_greenthread_spawns)

    def test_config_option_regex_match(self):
        def should_match(code):
            self.assertTrue(checks.cfg_opt_re.match(code))

        def should_not_match(code):
            self.assertFalse(checks.cfg_opt_re.match(code))

        should_match("opt = cfg.StrOpt('opt_name')")
        should_match("opt = cfg.IntOpt('opt_name')")
        should_match("opt = cfg.DictOpt('opt_name')")
        should_match("opt = cfg.Opt('opt_name')")
        should_match("opts=[cfg.Opt('opt_name')]")
        should_match("   cfg.Opt('opt_name')")
        should_not_match("opt_group = cfg.OptGroup('opt_group_name')")

    def test_check_config_option_in_central_place(self):
        errors = [(1, 0, "N342")]
        code = """
        opts = [
            cfg.StrOpt('random_opt',
                       default='foo',
                       help='I am here to do stuff'),
            ]
        """
        # option at the right place in the tree
        self._assert_has_no_errors(code,
                                   checks.check_config_option_in_central_place,
                                   filename="nova/conf/serial_console.py")
        # option at the wrong place in the tree
        self._assert_has_errors(code,
                                checks.check_config_option_in_central_place,
                                filename="nova/cmd/serialproxy.py",
                                expected_errors=errors)

        # option at a location which is marked as an exception
        # TODO(macsz) remove testing exceptions as they are removed from
        # check_config_option_in_central_place
        self._assert_has_no_errors(code,
                                   checks.check_config_option_in_central_place,
                                   filename="nova/cmd/manage.py")
        self._assert_has_no_errors(code,
                                   checks.check_config_option_in_central_place,
                                   filename="nova/tests/dummy_test.py")

    def test_check_doubled_words(self):
        errors = [(1, 0, "N343")]

        # Explicit addition of line-ending here and below since this isn't a
        # block comment and without it we trigger #1804062. Artificial break is
        # necessary to stop flake8 detecting the test
        code = "'This is the" + " the best comment'\n"
        self._assert_has_errors(code, checks.check_doubled_words,
                                expected_errors=errors)

        code = "'This is the then best comment'\n"
        self._assert_has_no_errors(code, checks.check_doubled_words)

    def test_no_os_popen(self):
        code = """
               import os

               foobar_cmd = "foobar -get -beer"
               answer = os.popen(foobar_cmd).read()

               if answer == nok":
                   try:
                       os.popen(os.popen('foobar -beer -please')).read()

                   except ValueError:
                       go_home()
               """
        errors = [(4, 0, 'N348'), (8, 8, 'N348')]
        self._assert_has_errors(code, checks.no_os_popen,
                                expected_errors=errors)

    def test_no_log_warn(self):
        code = """
                  LOG.warn("LOG.warn is deprecated")
               """
        errors = [(1, 0, 'N352')]
        self._assert_has_errors(code, checks.no_log_warn,
                                expected_errors=errors)
        code = """
                  LOG.warning("LOG.warn is deprecated")
               """
        self._assert_has_no_errors(code, checks.no_log_warn)

    def test_uncalled_closures(self):

        checker = checks.CheckForUncalledTestClosure
        code = """
               def test_fake_thing():
                   def _test():
                       pass
               """
        self._assert_has_errors(code, checker,
                expected_errors=[(1, 0, 'N349')])

        code = """
               def test_fake_thing():
                   def _test():
                       pass
                   _test()
               """
        self._assert_has_no_errors(code, checker)

        code = """
               def test_fake_thing():
                   def _test():
                       pass
                   self.assertRaises(FakeExcepion, _test)
               """
        self._assert_has_no_errors(code, checker)

    def test_check_policy_registration_in_central_place(self):
        errors = [(3, 0, "N350")]
        code = """
        from nova import policy

        policy.RuleDefault('context_is_admin', 'role:admin')
        """
        # registration in the proper place
        self._assert_has_no_errors(
            code, checks.check_policy_registration_in_central_place,
            filename="nova/policies/base.py")
        # option at a location which is not in scope right now
        self._assert_has_errors(
            code, checks.check_policy_registration_in_central_place,
            filename="nova/api/openstack/compute/non_existent.py",
            expected_errors=errors)

    def test_check_policy_enforce(self):
        errors = [(3, 0, "N351")]
        code = """
        from nova import policy

        policy._ENFORCER.enforce('context_is_admin', target, credentials)
        """
        self._assert_has_errors(code, checks.check_policy_enforce,
                                expected_errors=errors)

    def test_check_policy_enforce_does_not_catch_other_enforce(self):
        # Simulate a different enforce method defined in Nova
        code = """
        from nova import foo

        foo.enforce()
        """
        self._assert_has_no_errors(code, checks.check_policy_enforce)

    def test_log_context(self):
        code = """
                  LOG.info("Rebooting instance",
                           context=context, instance=instance)
               """
        errors = [(1, 0, 'N353')]
        self._assert_has_errors(code, checks.check_context_log,
                                expected_errors=errors)
        code = """
                  LOG.info("Rebooting instance",
                           context=admin_context, instance=instance)
               """
        errors = [(1, 0, 'N353')]
        self._assert_has_errors(code, checks.check_context_log,
                                expected_errors=errors)
        code = """
                  LOG.info("Rebooting instance", instance=instance)
               """
        self._assert_has_no_errors(code, checks.check_context_log)

    def test_no_assert_equal_true_false(self):
        code = """
                  self.assertEqual(context_is_admin, True)
                  self.assertEqual(context_is_admin, False)
                  self.assertEqual(True, context_is_admin)
                  self.assertEqual(False, context_is_admin)
                  self.assertNotEqual(context_is_admin, True)
                  self.assertNotEqual(context_is_admin, False)
                  self.assertNotEqual(True, context_is_admin)
                  self.assertNotEqual(False, context_is_admin)
               """
        errors = [(1, 0, 'N355'), (2, 0, 'N355'), (3, 0, 'N355'),
                  (4, 0, 'N355'), (5, 0, 'N355'), (6, 0, 'N355'),
                  (7, 0, 'N355'), (8, 0, 'N355')]
        self._assert_has_errors(code, checks.no_assert_equal_true_false,
                                expected_errors=errors)
        code = """
                  self.assertEqual(context_is_admin, stuff)
                  self.assertNotEqual(context_is_admin, stuff)
               """
        self._assert_has_no_errors(code, checks.no_assert_equal_true_false)

    def test_no_assert_true_false_is_not(self):
        code = """
                  self.assertTrue(test is None)
                  self.assertTrue(False is my_variable)
                  self.assertFalse(None is test)
                  self.assertFalse(my_variable is False)
               """
        errors = [(1, 0, 'N356'), (2, 0, 'N356'), (3, 0, 'N356'),
                  (4, 0, 'N356')]
        self._assert_has_errors(code, checks.no_assert_true_false_is_not,
                                expected_errors=errors)

    def test_check_uuid4(self):
        code = """
                  fake_uuid = uuid.uuid4()
                  hex_uuid = uuid.uuid4().hex
               """
        errors = [(1, 0, 'N357'), (2, 0, 'N357')]
        self._assert_has_errors(code, checks.check_uuid4,
                                expected_errors=errors)
        code = """
                  int_uuid = uuid.uuid4().int
                  urn_uuid = uuid.uuid4().urn
                  variant_uuid = uuid.uuid4().variant
                  version_uuid = uuid.uuid4().version
               """
        self._assert_has_no_errors(code, checks.check_uuid4)

    def test_return_followed_by_space(self):
        code = """
                  return(42)
                  return(a, b)
                  return(' some string ')
               """
        errors = [(1, 0, 'N358'), (2, 0, 'N358'), (3, 0, 'N358')]
        self._assert_has_errors(code, checks.return_followed_by_space,
                                expected_errors=errors)
        code = """
                  return
                  return 42
                  return (a, b)
                  return a, b
                  return ' some string '
               """
        self._assert_has_no_errors(code, checks.return_followed_by_space)

    def test_no_redundant_import_alias(self):
        code = """
                  from x import y as y
                  import x as x
                  import x.y.z as z
                  import x.y.z as y.z
               """
        errors = [(x + 1, 0, 'N359') for x in range(4)]
        self._assert_has_errors(code, checks.no_redundant_import_alias,
                                expected_errors=errors)
        code = """
                  from x import y
                  import x
                  from x.y import z
                  from a import bcd as cd
                  import ab.cd.efg as fg
                  import ab.cd.efg as d.efg
               """
        self._assert_has_no_errors(code, checks.no_redundant_import_alias)

    def test_yield_followed_by_space(self):
        code = """
                  yield(x, y)
                  yield{"type": "test"}
                  yield[a, b, c]
                  yield"test"
                  yield'test'
               """
        errors = [(x + 1, 0, 'N360') for x in range(5)]
        self._assert_has_errors(code, checks.yield_followed_by_space,
                                expected_errors=errors)
        code = """
                  yield x
                  yield (x, y)
                  yield {"type": "test"}
                  yield [a, b, c]
                  yield "test"
                  yield 'test'
                  yieldx_func(a, b)
               """
        self._assert_has_no_errors(code, checks.yield_followed_by_space)

    def test_assert_regexpmatches(self):
        code = """
                   self.assertRegexpMatches("Test", output)
                   self.assertNotRegexpMatches("Notmatch", output)
               """
        errors = [(x + 1, 0, 'N361') for x in range(2)]
        self._assert_has_errors(code, checks.assert_regexpmatches,
                                expected_errors=errors)
        code = """
                   self.assertRegexpMatchesfoo("Test", output)
                   self.assertNotRegexpMatchesbar("Notmatch", output)
               """
        self._assert_has_no_errors(code, checks.assert_regexpmatches)

    def test_import_alias_privsep(self):
        code = """
                  from nova import privsep
                  import nova.privsep as nova_privsep
                  from nova.privsep import linux_net
                  import nova.privsep.linux_net as privsep_linux_net
               """
        errors = [(x + 1, 0, 'N362') for x in range(4)]
        bad_filenames = ('nova/foo/bar.py',
                         'nova/foo/privsep.py',
                         'nova/privsep_foo/bar.py')
        for filename in bad_filenames:
            self._assert_has_errors(
                code, checks.privsep_imports_not_aliased,
                expected_errors=errors,
                filename=filename)
        good_filenames = ('nova/privsep.py',
                          'nova/privsep/__init__.py',
                          'nova/privsep/foo.py')
        for filename in good_filenames:
            self._assert_has_no_errors(
                code, checks.privsep_imports_not_aliased, filename=filename)
        code = """
                  import nova.privsep
                  import nova.privsep.foo
                  import nova.privsep.foo.bar
                  import nova.foo.privsep
                  import nova.foo.privsep.bar
                  import nova.tests.unit.whatever
               """
        for filename in (good_filenames + bad_filenames):
            self._assert_has_no_errors(
                code, checks.privsep_imports_not_aliased, filename=filename)

    def test_did_you_mean_tuple(self):
        code = """
                    if foo in (bar):
                    if foo in ('bar'):
                    if foo in (path.to.CONST_1234):
                    if foo in (
                        bar):
               """
        errors = [(x + 1, 0, 'N363') for x in range(4)]
        self._assert_has_errors(
            code, checks.did_you_mean_tuple, expected_errors=errors)
        code = """
                    def in(this_would_be_weird):
                        # A match in (any) comment doesn't count
                        if foo in (bar,)
                            or foo in ('bar',)
                            or foo in ("bar",)
                            or foo in (set1 + set2)
                            or foo in ("string continuations "
                                "are probably okay")
                            or foo in (method_call_should_this_work()):
               """
        self._assert_has_no_errors(code, checks.did_you_mean_tuple)

    def test_nonexistent_assertion_methods_and_attributes(self):
        code = """
                   mock_sample.called_once()
                   mock_sample.called_once_with(a, "TEST")
                   mock_sample.has_calls([mock.call(x), mock.call(y)])
                   mock_sample.mock_assert_called()
                   mock_sample.mock_assert_called_once()
                   mock_sample.mock_assert_called_with(a, "xxxx")
                   mock_sample.mock_assert_called_once_with(a, b)
                   mock_sample.mock_assert_any_call(1, 2, instance=instance)
                   sample.mock_assert_has_calls([mock.call(x), mock.call(y)])
                   mock_sample.mock_assert_not_called()
                   mock_sample.asser_called()
                   mock_sample.asser_called_once()
                   mock_sample.asser_called_with(a, "xxxx")
                   mock_sample.asser_called_once_with(a, b)
                   mock_sample.asser_any_call(1, 2, instance=instance)
                   mock_sample.asser_has_calls([mock.call(x), mock.call(y)])
                   mock_sample.asser_not_called()
                   mock_sample.asset_called()
                   mock_sample.asset_called_once()
                   mock_sample.asset_called_with(a, "xxxx")
                   mock_sample.asset_called_once_with(a, b)
                   mock_sample.asset_any_call(1, 2, instance=instance)
                   mock_sample.asset_has_calls([mock.call(x), mock.call(y)])
                   mock_sample.asset_not_called()
                   mock_sample.asssert_called()
                   mock_sample.asssert_called_once()
                   mock_sample.asssert_called_with(a, "xxxx")
                   mock_sample.asssert_called_once_with(a, b)
                   mock_sample.asssert_any_call(1, 2, instance=instance)
                   mock_sample.asssert_has_calls([mock.call(x), mock.call(y)])
                   mock_sample.asssert_not_called()
                   mock_sample.assset_called()
                   mock_sample.assset_called_once()
                   mock_sample.assset_called_with(a, "xxxx")
                   mock_sample.assset_called_once_with(a, b)
                   mock_sample.assset_any_call(1, 2, instance=instance)
                   mock_sample.assset_has_calls([mock.call(x), mock.call(y)])
                   mock_sample.assset_not_called()
                   mock_sample.retrun_value = 100
                   sample.call_method(mock_sample.retrun_value, 100)
                   mock.Mock(retrun_value=100)
               """
        errors = [(x + 1, 0, 'N364') for x in range(41)]
        # Check errors in 'nova/tests' directory.
        self._assert_has_errors(
            code, checks.nonexistent_assertion_methods_and_attributes,
            expected_errors=errors, filename="nova/tests/unit/test_context.py")
        # Check no errors in other than 'nova/tests' directory.
        self._assert_has_no_errors(
            code, checks.nonexistent_assertion_methods_and_attributes,
            filename="nova/compute/api.py")

        code = """
                   mock_sample.assert_called()
                   mock_sample.assert_called_once()
                   mock_sample.assert_called_with(a, "xxxx")
                   mock_sample.assert_called_once_with(a, b)
                   mock_sample.assert_any_call(1, 2, instance=instance)
                   mock_sample.assert_has_calls([mock.call(x), mock.call(y)])
                   mock_sample.assert_not_called()
                   mock_sample.return_value = 100
                   sample.call_method(mock_sample.return_value, 100)
                   mock.Mock(return_value=100)

                   sample.has_other_calls([1, 2, 3, 4])
                   sample.get_called_once("TEST")
                   sample.check_called_once_with("TEST")
                   mock_assert_method.assert_called()
                   test.asset_has_all(test)
                   test_retrun_value = 99
                   test.retrun_values = 100
               """
        self._assert_has_no_errors(
            code, checks.nonexistent_assertion_methods_and_attributes,
            filename="nova/tests/unit/test_context.py")

    def test_useless_assertion(self):
        code = """
                   self.assertIsNone(None, status)
                   self.assertTrue(True, flag)
                   self.assertTrue(10, count)
                   self.assertTrue('active', status)
                   self.assertTrue("building", status)
               """
        errors = [(x + 1, 0, 'N365') for x in range(5)]
        # Check errors in 'nova/tests' directory.
        self._assert_has_errors(
            code, checks.useless_assertion,
            expected_errors=errors, filename="nova/tests/unit/test_context.py")
        # Check no errors in other than 'nova/tests' directory.
        self._assert_has_no_errors(
            code, checks.useless_assertion,
            filename="nova/compute/api.py")
        code = """
                   self.assertIsNone(None_test_var, "Fails")
                   self.assertTrue(True_test_var, 'Fails')
                   self.assertTrue(var2, "Fails")
                   self.assertTrue(test_class.is_active('active'), 'Fails')
                   self.assertTrue(check_status("building"), 'Fails')
               """
        self._assert_has_no_errors(
            code, checks.useless_assertion,
            filename="nova/tests/unit/test_context.py")

    def test_check_assert_has_calls(self):
        code = """
                   mock_method.assert_has_calls = [mock.call(1)]
                   mock_method2.assert_has_calls = [
                       mock.call(1), mock.call(2)]
               """
        errors = [(x + 1, 0, 'N366') for x in range(2)]
        # Check errors in 'nova/tests' directory.
        self._assert_has_errors(
            code, checks.check_assert_has_calls,
            expected_errors=errors, filename="nova/tests/unit/test_context.py")
        # Check no errors in other than 'nova/tests' directory.
        self._assert_has_no_errors(
            code, checks.check_assert_has_calls,
            filename="nova/compute/api.py")
        code = """
                   mock_method.assert_has_calls([mock.call(1)])
                   mock_method2.assert_has_calls([
                       mock.call(1), mock.call(2)])
               """
        self._assert_has_no_errors(
            code, checks.check_assert_has_calls,
            filename="nova/tests/unit/test_context.py")

    def test_do_not_alias_mock_class(self):
        code = """
                    my_var = mock.Mock
                    self.mock_rados.Rados = mock.Mock
                    self.mock_rados.Rados = mock.MagicMock
                    self.mock_rados.Rados = mock.NonCallableMock
               """
        errors = [(x + 1, 0, 'N367') for x in range(4)]
        # Check errors in 'nova/tests' directory.
        self._assert_has_errors(
            code, checks.do_not_alias_mock_class,
            expected_errors=errors, filename="nova/tests/unit/test_context.py")
        # Check no errors in other than 'nova/tests' directory.
        self._assert_has_no_errors(
            code, checks.do_not_alias_mock_class,
            filename="nova/compute/api.py")
        code = """
                    my_var = mock.Mock()
                    self.mock_rados.Rados = mock.Mock()
                    self.mock_rados.Rados = mock.MagicMock()
                    self.mock_rados.Rados = mock.NonCallableMock()
               """
        self._assert_has_no_errors(
            code, checks.do_not_alias_mock_class,
            filename="nova/tests/unit/test_context.py")

    def test_do_not_use_mock_class_as_new_mock_value(self):
        code = """
                    patcher = mock.patch('Bar.foo', new=mock.Mock)
                    patcher = mock.patch.object(Bar, 'foo', new=mock.Mock)
                    patcher = mock.patch('Bar.foo', new=mock.MagicMock)
                    patcher = mock.patch('Bar.foo', new=mock.NonCallableMock)
                    @mock.patch('Bar.foo', new=mock.Mock)
                    def a():
                        pass

                    with mock.patch('Bar.foo', new=mock.Mock) as m:
                        pass
               """
        errors = [(x + 1, 0, 'N368') for x in range(5)] + [(9, 0, 'N368')]
        # Check errors in 'nova/tests' directory.
        self._assert_has_errors(
            code, checks.do_not_use_mock_class_as_new_mock_value,
            expected_errors=errors, filename="nova/tests/unit/test_context.py")
        # Check no errors in other than 'nova/tests' directory.
        self._assert_has_no_errors(
            code, checks.do_not_use_mock_class_as_new_mock_value,
            filename="nova/compute/api.py")
        code = """
                    patcher = mock.patch('Bar.foo', new=mock.Mock())
                    patcher = mock.patch.object(Bar, 'foo', new=mock.Mock())
                    patcher = mock.patch('Bar.foo', new=mock.MagicMock())
                    patcher = mock.patch('Bar.foo', new=mock.NonCallableMock())
                    @mock.patch('Bar.foo', new=mock.Mock())
                    def a():
                        pass

                    with mock.patch('Bar.foo', new=mock.Mock()) as m:
                        pass

                    patcher = mock.patch('Bar.foo', new_callable=mock.Mock)
               """
        self._assert_has_no_errors(
            code, checks.do_not_use_mock_class_as_new_mock_value,
            filename="nova/tests/unit/test_context.py")

    def test_check_lockutils_rwlocks(self):
        code = """
                    lockutils.ReaderWriterLock()
                    lockutils.ReaderWriterLock(condition_cls=MyClass)
                    oslo_concurrency.lockutils.ReaderWriterLock()
                    fasteners.ReaderWriterLock()
                    fasteners.ReaderWriterLock(condition_cls=MyClass)
               """
        errors = [(x + 1, 0, 'N369') for x in range(5)]
        self._assert_has_errors(
            code, checks.check_lockutils_rwlocks, expected_errors=errors)

        code = """
                    nova.utils.ReaderWriterLock()
                    utils.ReaderWriterLock()
                    utils.ReaderWriterLock(condition_cls=MyClass)
                    nova_utils.ReaderWriterLock()
               """
        self._assert_has_no_errors(code, checks.check_lockutils_rwlocks)

    def test_check_six(self):
        code = """
            import six
            from six import moves
            from six.moves import range
            import six.moves.urllib.parse as urlparse
        """
        errors = [(x + 1, 0, 'N370') for x in range(4)]
        self._assert_has_errors(code, checks.check_six, expected_errors=errors)

    def test_import_stock_mock(self):
        self._assert_has_errors(
            "import mock",
            checks.import_stock_mock, expected_errors=[(1, 0, 'N371')])
        self._assert_has_errors(
            "from mock import patch",
            checks.import_stock_mock, expected_errors=[(1, 0, 'N371')])
        code = """
                    from unittest import mock
                    import unittest.mock
               """
        self._assert_has_no_errors(code, checks.import_stock_mock)

    def test_check_set_daemon(self):
        code = """
                   self.setDaemon(True)
                   worker.setDaemon(True)
                   self._event_thread.setDaemon(True)
                   mythread.setDaemon(False)
                   self.thread.setDaemon(1)
               """
        errors = [(x + 1, 0, 'N372') for x in range(5)]
        self._assert_has_errors(
            code, checks.check_set_daemon, expected_errors=errors)

        code = """
                   self.setDaemon = True
                   worker.setDaemonFlag(True)
                   self._event_thread.resetDaemon(True)
                   self.set.Daemon(True)
                   self.thread.setdaemon(True)
               """
        self._assert_has_no_errors(code, checks.check_set_daemon)



# Copyright 2016 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.


force_complete = {
    'type': 'object',
    'properties': {
        'force_complete': {
            'type': 'null'
        }
    },
    'required': ['force_complete'],
    'additionalProperties': False,
}



# Copyright (c) 2013 Intel, Inc.
# Copyright (c) 2013 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from oslo_config import cfg

pci_group = cfg.OptGroup(
    name='pci',
    title='PCI passthrough options')

pci_opts = [
    cfg.MultiStrOpt('alias',
        default=[],
        deprecated_name='pci_alias',
        deprecated_group='DEFAULT',
        help="""
An alias for a PCI passthrough device requirement.

This allows users to specify the alias in the extra specs for a flavor, without
needing to repeat all the PCI property requirements.

This should be configured for the ``nova-api`` service and, assuming you wish
to use move operations, for each ``nova-compute`` service.

Possible Values:

* A dictionary of JSON values which describe the aliases. For example::

    alias = {
      "name": "QuickAssist",
      "product_id": "0443",
      "vendor_id": "8086",
      "device_type": "type-PCI",
      "numa_policy": "required"
    }

  This defines an alias for the Intel QuickAssist card. (multi valued). Valid
  key values are :

  ``name``
    Name of the PCI alias.

  ``product_id``
    Product ID of the device in hexadecimal.

  ``vendor_id``
    Vendor ID of the device in hexadecimal.

  ``device_type``
    Type of PCI device. Valid values are: ``type-PCI``, ``type-PF`` and
    ``type-VF``. Note that ``"device_type": "type-PF"`` **must** be specified
    if you wish to passthrough a device that supports SR-IOV in its entirety.

  ``numa_policy``
    Required NUMA affinity of device. Valid values are: ``legacy``,
    ``preferred`` and ``required``.

  ``resource_class``
    The optional Placement resource class name that is used
    to track the requested PCI devices in Placement. It can be a standard
    resource class from the ``os-resource-classes`` lib. Or it can be an
    arbitrary string. If it is an non-standard resource class then Nova will
    normalize it to a proper Placement resource class by
    making it upper case, replacing any consecutive character outside of
    ``[A-Z0-9_]`` with a single '_', and prefixing the name with ``CUSTOM_`` if
    not yet prefixed. The maximum allowed length is 255 character including the
    prefix. If ``resource_class`` is not provided Nova will generate it from
    ``vendor_id`` and ``product_id`` values of the alias in the form of
    ``CUSTOM_PCI_{vendor_id}_{product_id}``. The ``resource_class`` requested
    in the alias is matched against the ``resource_class`` defined in the
    ``[pci]device_spec``. This field can only be used only if
    ``[filter_scheduler]pci_in_placement`` is enabled.

  ``traits``
    An optional comma separated list of Placement trait names requested to be
    present on the resource provider that fulfills this alias. Each trait can
    be a standard trait from ``os-traits`` lib or it can be an arbitrary
    string. If it is a non-standard trait then Nova will normalize the
    trait name by making it upper case, replacing any consecutive character
    outside of  ``[A-Z0-9_]`` with a single '_', and  prefixing the name
    with ``CUSTOM_`` if not yet prefixed. The maximum allowed length of a
    trait name is 255 character including the prefix. Every trait in
    ``traits`` requested in the alias ensured to be in the list of traits
    provided in the ``traits`` field of the ``[pci]device_spec`` when
    scheduling the request. This field can only be used only if
    ``[filter_scheduler]pci_in_placement`` is enabled.

* Supports multiple aliases by repeating the option (not by specifying
  a list value)::

    alias = {
      "name": "QuickAssist-1",
      "product_id": "0443",
      "vendor_id": "8086",
      "device_type": "type-PCI",
      "numa_policy": "required"
    }
    alias = {
      "name": "QuickAssist-2",
      "product_id": "0444",
      "vendor_id": "8086",
      "device_type": "type-PCI",
      "numa_policy": "required"
    }
"""),
    cfg.MultiStrOpt('device_spec',
        default=[],
        deprecated_opts=[
            cfg.DeprecatedOpt('passthrough_whitelist', group='pci'),
            cfg.DeprecatedOpt('pci_passthrough_whitelist', group='DEFAULT'),
        ],
        help="""
Specify the PCI devices available to VMs.

Possible values:

* A JSON dictionary which describe a PCI device. It should take
  the following format::

    ["vendor_id": "<id>",] ["product_id": "<id>",]
    ["address": "[[[[<domain>]:]<bus>]:][<slot>][.[<function>]]" |
     "devname": "<name>",]
    {"<tag>": "<tag_value>",}

  Where ``[`` indicates zero or one occurrences, ``{`` indicates zero or
  multiple occurrences, and ``|`` mutually exclusive options. Note that any
  missing fields are automatically wildcarded.

  Valid key values are :

  ``vendor_id``
    Vendor ID of the device in hexadecimal.

  ``product_id``
    Product ID of the device in hexadecimal.

  ``address``
    PCI address of the device. Both traditional glob style and regular
    expression syntax is supported. Please note that the address fields are
    restricted to the following maximum values:

    * domain - 0xFFFF
    * bus - 0xFF
    * slot - 0x1F
    * function - 0x7

  ``devname``
    Device name of the device (for e.g. interface name). Not all PCI devices
    have a name.

  ``<tag>``
    Additional ``<tag>`` and ``<tag_value>`` used for specifying PCI devices.
    Supported ``<tag>`` values are :

    - ``physical_network``
    - ``trusted``
    - ``remote_managed`` - a VF is managed remotely by an off-path networking
      backend. May have boolean-like string values case-insensitive values:
      "true" or "false". By default, "false" is assumed for all devices.
      Using this option requires a networking service backend capable of
      handling those devices. PCI devices are also required to have a PCI
      VPD capability with a card serial number (either on a VF itself on
      its corresponding PF), otherwise they will be ignored and not
      available for allocation.
    - ``resource_class`` - optional Placement resource class name to be used
      to track the matching PCI devices in Placement when [pci]device_spec is
      True. It can be a standard resource class from the
      ``os-resource-classes`` lib. Or can be any string. In that case Nova will
      normalize it to a proper Placement resource class by making it upper
      case, replacing any consecutive character outside of ``[A-Z0-9_]`` with a
      single '_', and prefixing the name with ``CUSTOM_`` if not yet prefixed.
      The maximum allowed length is 255 character including the prefix.
      If ``resource_class`` is not provided Nova will generate it from the PCI
      device's ``vendor_id`` and ``product_id`` in the form of
      ``CUSTOM_PCI_{vendor_id}_{product_id}``.
      The ``resource_class`` can be requested from a ``[pci]alias``
    - ``traits`` - optional comma separated list of Placement trait names to
      report on the resource provider that will represent the matching PCI
      device. Each trait can be a standard trait from ``os-traits`` lib or can
      be any string. If it is not a standard trait then Nova will normalize the
      trait name by making it upper case, replacing any consecutive character
      outside of  ``[A-Z0-9_]`` with a single '_', and  prefixing the name with
      ``CUSTOM_`` if not yet prefixed. The maximum allowed length of a trait
      name is 255 character including the prefix.
      Any trait from ``traits`` can be requested from a ``[pci]alias``.


  Valid examples are::

    device_spec = {"devname":"eth0",
                   "physical_network":"physnet"}
    device_spec = {"address":"*:0a:00.*"}
    device_spec = {"address":":0a:00.",
                   "physical_network":"physnet1"}
    device_spec = {"vendor_id":"1137",
                   "product_id":"0071"}
    device_spec = {"vendor_id":"1137",
                   "product_id":"0071",
                   "address": "0000:0a:00.1",
                   "physical_network":"physnet1"}
    device_spec = {"address":{"domain": ".*",
                              "bus": "02", "slot": "01",
                              "function": "[2-7]"},
                   "physical_network":"physnet1"}
    device_spec = {"address":{"domain": ".*",
                              "bus": "02", "slot": "0[1-2]",
                              "function": ".*"},
                   "physical_network":"physnet1"}
    device_spec = {"devname": "eth0", "physical_network":"physnet1",
                   "trusted": "true"}
    device_spec = {"vendor_id":"a2d6",
                   "product_id":"15b3",
                   "remote_managed": "true"}
    device_spec = {"vendor_id":"a2d6",
                   "product_id":"15b3",
                   "address": "0000:82:00.0",
                   "physical_network":"physnet1",
                   "remote_managed": "true"}
    device_spec = {"vendor_id":"1002",
                   "product_id":"6929",
                   "address": "0000:82:00.0",
                   "resource_class": "PGPU",
                   "traits": "HW_GPU_API_VULKAN,my-awesome-gpu"}

  The following are invalid, as they specify mutually exclusive options::

    device_spec = {"devname":"eth0",
                   "physical_network":"physnet",
                   "address":"*:0a:00.*"}

  The following example is invalid because it specifies the ``remote_managed``
  tag for a PF - it will result in an error during config validation at the
  Nova Compute service startup::

    device_spec = {"address": "0000:82:00.0",
                   "product_id": "a2d6",
                   "vendor_id": "15b3",
                   "physical_network": null,
                   "remote_managed": "true"}

* A JSON list of JSON dictionaries corresponding to the above format. For
  example::

    device_spec = [{"product_id":"0001", "vendor_id":"8086"},
                   {"product_id":"0002", "vendor_id":"8086"}]
"""),
    cfg.BoolOpt('report_in_placement',
                default=False,
                help="""
Enable PCI resource inventory reporting to Placement. If it is enabled then the
nova-compute service will report PCI resource inventories to Placement
according to the [pci]device_spec configuration and the PCI devices reported
by the hypervisor. Once it is enabled it cannot be disabled any more. In a
future release the default of this config will be change to True.

Related options:

* [pci]device_spec: to define which PCI devices nova are allowed to track and
  assign to guests.
"""),
]


def register_opts(conf):
    conf.register_group(pci_group)
    conf.register_opts(pci_opts, group=pci_group)


def list_opts():
    return {pci_group: pci_opts}



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import os
from unittest import mock

from nova import exception
from nova import filesystem
from nova import test


class TestFSCommon(test.NoDBTestCase):

    def test_read_sys(self):
        open_mock = mock.mock_open(read_data='bar')
        with mock.patch('builtins.open', open_mock) as m_open:
            self.assertEqual('bar', filesystem.read_sys('foo'))
        expected_path = os.path.join(filesystem.SYS, 'foo')
        m_open.assert_called_once_with(expected_path, mode='r')

    def test_read_sys_error(self):
        with mock.patch('builtins.open',
                        side_effect=OSError('error')) as m_open:
            self.assertRaises(exception.FileNotFound,
                              filesystem.read_sys, 'foo')
        expected_path = os.path.join(filesystem.SYS, 'foo')
        m_open.assert_called_once_with(expected_path, mode='r')

    def test_write_sys(self):
        open_mock = mock.mock_open()
        with mock.patch('builtins.open', open_mock) as m_open:
            self.assertIsNone(filesystem.write_sys('foo', 'bar'))
        expected_path = os.path.join(filesystem.SYS, 'foo')
        m_open.assert_called_once_with(expected_path, mode='w')
        open_mock().write.assert_called_once_with('bar')

    def test_write_sys_error(self):
        with mock.patch('builtins.open',
                        side_effect=OSError('fake_error')) as m_open:
            self.assertRaises(exception.FileNotFound,
                              filesystem.write_sys, 'foo', 'bar')
        expected_path = os.path.join(filesystem.SYS, 'foo')
        m_open.assert_called_once_with(expected_path, mode='w')



# Copyright 2011 Grid Dynamics
# Copyright 2011 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import copy

from oslo_utils.fixture import uuidsentinel as uuids
import webob

from nova.api.openstack.compute import networks as networks_v21
from nova import exception
from nova.network import neutron
from nova import test
from nova.tests.unit.api.openstack import fakes


# NOTE(stephenfin): obviously these aren't complete responses, but this is all
# we care about
FAKE_NETWORKS = [
    {
        'id': uuids.private,
        'name': 'private',
    },
    {
        'id': uuids.public,
        'name': 'public',
    },
]

USER_RESPONSE_TEMPLATE = {
    field: None for field in (
        'id', 'cidr', 'netmask', 'gateway', 'broadcast', 'dns1', 'dns2',
        'cidr_v6', 'gateway_v6', 'label', 'netmask_v6')
}

ADMIN_RESPONSE_TEMPLATE = {
    field: None for field in (
        'id', 'cidr', 'netmask', 'gateway', 'broadcast', 'dns1', 'dns2',
        'cidr_v6', 'gateway_v6', 'label', 'netmask_v6', 'created_at',
        'updated_at', 'deleted_at', 'deleted', 'injected', 'bridge', 'vlan',
        'vpn_public_address', 'vpn_public_port', 'vpn_private_address',
        'dhcp_start', 'project_id', 'host', 'bridge_interface', 'multi_host',
        'priority', 'rxtx_base', 'mtu', 'dhcp_server', 'enable_dhcp',
        'share_address')
}


class FakeNetworkAPI(object):

    def __init__(self):
        self.networks = copy.deepcopy(FAKE_NETWORKS)

    def get_all(self, context):
        return self.networks

    def get(self, context, network_id):
        for network in self.networks:
            if network['id'] == network_id:
                return network
        raise exception.NetworkNotFound(network_id=network_id)


class NetworksTestV21(test.NoDBTestCase):
    validation_error = exception.ValidationError

    def setUp(self):
        super(NetworksTestV21, self).setUp()
        # TODO(stephenfin): Consider using then NeutronFixture here
        self.fake_network_api = FakeNetworkAPI()
        self._setup()
        fakes.stub_out_networking(self)
        self.non_admin_req = fakes.HTTPRequest.blank(
            '', project_id=fakes.FAKE_PROJECT_ID)
        self.admin_req = fakes.HTTPRequest.blank(
            '', project_id=fakes.FAKE_PROJECT_ID, use_admin_context=True)

    def _setup(self):
        self.controller = networks_v21.NetworkController(
            self.fake_network_api)
        self.neutron_ctrl = networks_v21.NetworkController(
            neutron.API())
        self.req = fakes.HTTPRequest.blank('',
                                           project_id=fakes.FAKE_PROJECT_ID)

    def _check_status(self, res, method, code):
        self.assertEqual(method.wsgi_code, code)

    def test_network_list_all_as_user(self):
        res_dict = self.controller.index(self.non_admin_req)

        expected = []
        for fake_network in FAKE_NETWORKS:
            expected_ = copy.deepcopy(USER_RESPONSE_TEMPLATE)
            expected_['id'] = fake_network['id']
            expected_['label'] = fake_network['name']
            expected.append(expected_)

        self.assertEqual({'networks': expected}, res_dict)

    def test_network_list_all_as_admin(self):
        res_dict = self.controller.index(self.admin_req)

        expected = []
        for fake_network in FAKE_NETWORKS:
            expected_ = copy.deepcopy(ADMIN_RESPONSE_TEMPLATE)
            expected_['id'] = fake_network['id']
            expected_['label'] = fake_network['name']
            expected.append(expected_)

        self.assertEqual({'networks': expected}, res_dict)

    def test_network_get_as_user(self):
        uuid = FAKE_NETWORKS[0]['id']
        res_dict = self.controller.show(self.non_admin_req, uuid)

        expected = copy.deepcopy(USER_RESPONSE_TEMPLATE)
        expected['id'] = FAKE_NETWORKS[0]['id']
        expected['label'] = FAKE_NETWORKS[0]['name']

        self.assertEqual({'network': expected}, res_dict)

    def test_network_get_as_admin(self):
        uuid = FAKE_NETWORKS[0]['id']
        res_dict = self.controller.show(self.admin_req, uuid)

        expected = copy.deepcopy(ADMIN_RESPONSE_TEMPLATE)
        expected['id'] = FAKE_NETWORKS[0]['id']
        expected['label'] = FAKE_NETWORKS[0]['name']

        self.assertEqual({'network': expected}, res_dict)

    def test_network_get_not_found(self):
        self.assertRaises(webob.exc.HTTPNotFound,
                          self.controller.show, self.req, uuids.invalid)


class NetworksDeprecationTest(test.NoDBTestCase):

    def setUp(self):
        super(NetworksDeprecationTest, self).setUp()
        self.controller = networks_v21.NetworkController()
        self.req = fakes.HTTPRequest.blank('', version='2.36')

    def test_all_api_return_not_found(self):
        self.assertRaises(exception.VersionNotFoundForAPIMethod,
            self.controller.show, self.req, fakes.FAKE_UUID)
        self.assertRaises(exception.VersionNotFoundForAPIMethod,
            self.controller.index, self.req)



# Copyright 2013 Cloudbase Solutions Srl
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import webob

from nova.api.openstack import wsgi
import nova.conf
from nova import context as nova_context
from nova.i18n import _
from nova import objects
from nova.policies import console_auth_tokens as cat_policies

CONF = nova.conf.CONF


class ConsoleAuthTokensController(wsgi.Controller):

    def _show(self, req, id):
        """Checks a console auth token and returns the related connect info."""
        context = req.environ['nova.context']
        context.can(cat_policies.BASE_POLICY_NAME)

        token = id
        if not token:
            msg = _("token not provided")
            raise webob.exc.HTTPBadRequest(explanation=msg)

        connect_info = None

        results = nova_context.scatter_gather_skip_cell0(
            context, objects.ConsoleAuthToken.validate, token)
        # NOTE(melwitt): Console token auths are stored in cell databases,
        # but with only the token as a request param, we can't know which
        # cell database contains the token's corresponding connection info.
        # So, we must query all cells for the info and we can break the
        # loop as soon as we find a result because the token is associated
        # with one instance, which can only be in one cell.
        for result in results.values():
            if not nova_context.is_cell_failure_sentinel(result):
                connect_info = result
                break

        if not connect_info:
            raise webob.exc.HTTPNotFound(explanation=_("Token not found"))

        return {'console': {
            'instance_uuid': connect_info.instance_uuid,
            'host': connect_info.host,
            'port': connect_info.port,
            'internal_access_path': connect_info.internal_access_path,
        }}

    @wsgi.Controller.api_version("2.1", "2.30")
    @wsgi.expected_errors((400, 401, 404))
    def show(self, req, id):
        """Until microversion 2.30, this API was available only for the
        rdp-html5 console type which has been removed along with the HyperV
        driver in the Nova 29.0.0 (Caracal) release. As this method is for
        microversion <=2.30, it will return an http 400 error. Starting
        from 2.31 microversion, this API works for all the supported
        console types that are handled by the separate show method
        defined below.
        """
        raise webob.exc.HTTPBadRequest()

    @wsgi.Controller.api_version("2.31")  # noqa
    @wsgi.expected_errors((400, 404))
    def show(self, req, id):  # noqa
        return self._show(req, id)
